
	.extra-feedback.svelte-fvat56 {
		display: flex;
		align-items: center;
		position: relative;
	}
	.extra-feedback-options.svelte-fvat56 {
		display: none;
		position: absolute;
		padding: var(--spacing-md) 0;
		flex-direction: column;
		gap: var(--spacing-sm);
		top: 100%;
	}
	.extra-feedback.svelte-fvat56:hover .extra-feedback-options:where(.svelte-fvat56) {
		display: flex;
	}
	.extra-feedback-option.svelte-fvat56 {
		border: 1px solid var(--border-color-primary);
		border-radius: var(--radius-sm);
		color: var(--block-label-text-color);
		background-color: var(--block-background-fill);
		font-size: var(--text-xs);
		padding: var(--spacing-xxs) var(--spacing-sm);
		width: max-content;
	}
	.bubble.svelte-704d7y .icon-button-wrapper {
		margin: 0px calc(var(--spacing-xl) * 2);
	}

	.message-buttons.svelte-704d7y {
		z-index: var(--layer-1);
	}
	.message-buttons-left.svelte-704d7y {
		align-self: flex-start;
	}

	.bubble.message-buttons-right.svelte-704d7y {
		align-self: flex-end;
	}

	.message-buttons-right.svelte-704d7y .icon-button-wrapper {
		margin-left: auto;
	}

	.bubble.with-avatar.svelte-704d7y {
		margin-left: calc(var(--spacing-xl) * 5);
		margin-right: calc(var(--spacing-xl) * 5);
	}

	.panel.svelte-704d7y {
		display: flex;
		align-self: flex-start;
		z-index: var(--layer-1);
	}
	.file-container.svelte-e5gd5s {
		display: flex;
		align-items: center;
		gap: var(--spacing-lg);
		padding: var(--spacing-lg);
		border-radius: var(--radius-lg);
		width: fit-content;
		margin: var(--spacing-sm) 0;
	}

	.file-icon.svelte-e5gd5s {
		display: flex;
		align-items: center;
		justify-content: center;
		color: var(--body-text-color);
	}

	.file-icon.svelte-e5gd5s svg {
		width: var(--size-7);
		height: var(--size-7);
	}

	.file-info.svelte-e5gd5s {
		display: flex;
		flex-direction: column;
	}

	.file-link.svelte-e5gd5s {
		text-decoration: none;
		color: var(--body-text-color);
		display: flex;
		flex-direction: column;
		gap: var(--spacing-xs);
	}

	.file-name.svelte-e5gd5s {
		font-family: var(--font);
		font-size: var(--text-md);
		font-weight: 500;
	}

	.file-type.svelte-e5gd5s {
		font-family: var(--font);
		font-size: var(--text-sm);
		color: var(--body-text-color-subdued);
		text-transform: uppercase;
	}
	.thought-group.svelte-18cn3o3 {
		background: var(--background-fill-primary);
		border: 1px solid var(--border-color-primary);
		border-radius: var(--radius-sm);
		padding: var(--spacing-md);
		margin: var(--spacing-md) 0;
		font-size: var(--text-sm);
	}

	.children.svelte-18cn3o3 .thought-group {
		border: none;
		margin: 0;
		padding-bottom: 0;
	}

	.children.svelte-18cn3o3 {
		padding-left: var(--spacing-md);
	}

	.title.svelte-18cn3o3 {
		display: flex;
		align-items: center;
		color: var(--body-text-color);
		cursor: pointer;
		width: 100%;
	}

	.title.svelte-18cn3o3 .md {
		font-size: var(--text-sm) !important;
	}

	.content.svelte-18cn3o3,
	.content-preview.svelte-18cn3o3 {
		overflow-wrap: break-word;
		word-break: break-word;
		margin-left: var(--spacing-lg);
		margin-bottom: var(--spacing-sm);
	}

	.content-preview.svelte-18cn3o3 {
		position: relative;
		max-height: calc(5 * 1.5em);
		overflow-y: auto;
		overscroll-behavior: contain;
		cursor: default;
	}

	.content.svelte-18cn3o3 *,
	.content-preview.svelte-18cn3o3 * {
		font-size: var(--text-sm);
		color: var(--body-text-color);
	}

	.thought-group.svelte-18cn3o3 .thought:not(.nested) {
		border: none;
		background: none;
	}

	.duration.svelte-18cn3o3 {
		color: var(--body-text-color-subdued);
		font-size: var(--text-sm);
		margin-left: var(--size-1);
	}

	.arrow.svelte-18cn3o3 {
		opacity: 0.8;
		width: var(--size-8);
		height: var(--size-8);
		display: flex;
		align-items: center;
		justify-content: center;
	}

	.arrow.svelte-18cn3o3 button {
		background-color: transparent;
	}

	.loading-spinner.svelte-18cn3o3 {
		display: inline-block;
		width: 12px;
		height: 12px;
		border: 2px solid var(--body-text-color);
		border-radius: 50%;
		border-top-color: transparent;
		animation: svelte-18cn3o3-spin 1s linear infinite;
		margin: 0 var(--size-1) -1px var(--size-2);
		opacity: 0.8;
	}

	@keyframes svelte-18cn3o3-spin {
		to {
			transform: rotate(360deg);
		}
	}

	.thought-group.svelte-18cn3o3 .message-content {
		opacity: 0.8;
	}
	.message.svelte-1nr59td {
		position: relative;
		width: 100%;
	}

	.message.display_consecutive_in_same_bubble.svelte-1nr59td {
		margin-top: 0;
	}

	.message.svelte-1nr59td + .message:where(.svelte-1nr59td) {
		margin-top: var(--spacing-sm);
	}

	/* avatar styles */
	.avatar-container.svelte-1nr59td {
		flex-shrink: 0;
		border-radius: 50%;
		border: 1px solid var(--border-color-primary);
		overflow: hidden;
	}

	.avatar-container.svelte-1nr59td img {
		object-fit: cover;
	}

	/* message wrapper */
	.flex-wrap.svelte-1nr59td {
		display: flex;
		flex-direction: column;
		width: calc(100% - var(--spacing-xxl));
		max-width: 100%;
		color: var(--body-text-color);
		font-size: var(--chatbot-text-size);
		overflow-wrap: break-word;
		width: 100%;
		height: 100%;
	}

	.component.svelte-1nr59td {
		padding: 0;
		border-radius: var(--radius-md);
		width: fit-content;
		overflow: hidden;
	}

	.component.gallery.svelte-1nr59td {
		border: none;
	}

	.bot.svelte-1nr59td:has(.model3D:where(.svelte-1nr59td)),
	.user.svelte-1nr59td:has(.model3D:where(.svelte-1nr59td)) {
		border: none;
		max-width: 75%;
	}

	.message-row.svelte-1nr59td :where(.svelte-1nr59td):not(.avatar-container) img {
		margin: var(--size-2);
		max-height: 300px;
	}

	.file-pil.svelte-1nr59td {
		display: block;
		width: fit-content;
		padding: var(--spacing-sm) var(--spacing-lg);
		border-radius: var(--radius-md);
		background: var(--background-fill-secondary);
		color: var(--body-text-color);
		text-decoration: none;
		margin: 0;
		font-family: var(--font-mono);
		font-size: var(--text-sm);
	}

	.file.svelte-1nr59td {
		width: auto !important;
		max-width: fit-content !important;
	}

	@media (max-width: 600px) or (max-width: 480px) {
		.component.svelte-1nr59td {
			width: 100%;
		}
	}

	.message.svelte-1nr59td .prose {
		font-size: var(--chatbot-text-size);
	}

	.message-bubble-border.svelte-1nr59td {
		border-width: 1px;
		border-radius: var(--radius-md);
	}

	.panel-full-width.svelte-1nr59td {
		width: 100%;
	}
	.message-markdown-disabled.svelte-1nr59td {
		white-space: pre-line;
	}

	.user.svelte-1nr59td {
		border-radius: var(--radius-md);
		align-self: flex-end;
		border-bottom-right-radius: 0;
		box-shadow: var(--shadow-drop);
		border: 1px solid var(--border-color-accent-subdued);
		background-color: var(--color-accent-soft);
		padding: var(--spacing-sm) var(--spacing-xl);
	}

	.bot.svelte-1nr59td {
		border: 1px solid var(--border-color-primary);
		border-radius: var(--radius-md);
		border-color: var(--border-color-primary);
		background-color: var(--background-fill-secondary);
		box-shadow: var(--shadow-drop);
		align-self: flex-start;
		text-align: left;
		border-bottom-left-radius: 0;
		padding: var(--spacing-sm) var(--spacing-xl);
	}

	.bot.svelte-1nr59td:has(.table-wrap:where(.svelte-1nr59td)) {
		border: none;
		box-shadow: none;
		background: none;
	}

	.panel.svelte-1nr59td .user:where(.svelte-1nr59td) * {
		text-align: right;
	}

	/* Colors */

	.message-row.svelte-1nr59td {
		display: flex;
		position: relative;
	}

	/* bubble mode styles */
	.bubble.svelte-1nr59td {
		margin: calc(var(--spacing-xl) * 2);
		margin-bottom: var(--spacing-xl);
	}

	.bubble.user-row.svelte-1nr59td {
		align-self: flex-end;
		max-width: calc(100% - var(--spacing-xl) * 6);
	}

	.bubble.bot-row.svelte-1nr59td {
		align-self: flex-start;
		max-width: calc(100% - var(--spacing-xl) * 6);
	}

	.bubble.svelte-1nr59td .user-row:where(.svelte-1nr59td) {
		flex-direction: row;
		justify-content: flex-end;
	}

	.bubble.svelte-1nr59td .with_avatar.user-row:where(.svelte-1nr59td) {
		margin-right: calc(var(--spacing-xl) * 2) !important;
	}

	.bubble.svelte-1nr59td .with_avatar.bot-row:where(.svelte-1nr59td) {
		margin-left: calc(var(--spacing-xl) * 2) !important;
	}

	.bubble.svelte-1nr59td .with_opposite_avatar.user-row:where(.svelte-1nr59td) {
		margin-left: calc(var(--spacing-xxl) + 35px + var(--spacing-xxl));
	}

	/* panel mode styles */
	.panel.svelte-1nr59td {
		margin: 0;
		padding: calc(var(--spacing-lg) * 2) calc(var(--spacing-lg) * 2);
	}

	.panel.bot-row.svelte-1nr59td {
		background: var(--background-fill-secondary);
	}

	.panel.svelte-1nr59td .with_avatar:where(.svelte-1nr59td) {
		padding-left: calc(var(--spacing-xl) * 2) !important;
		padding-right: calc(var(--spacing-xl) * 2) !important;
	}

	.panel.svelte-1nr59td .panel-full-width:where(.svelte-1nr59td) {
		width: 100%;
	}

	.panel.svelte-1nr59td .user:where(.svelte-1nr59td) * {
		text-align: right;
	}

	/* message content */
	.flex-wrap.svelte-1nr59td {
		display: flex;
		flex-direction: column;
		max-width: 100%;
		color: var(--body-text-color);
		font-size: var(--chatbot-text-size);
		overflow-wrap: break-word;
	}

	@media (max-width: 480px) {
		.user-row.bubble.svelte-1nr59td {
			align-self: flex-end;
		}

		.bot-row.bubble.svelte-1nr59td {
			align-self: flex-start;
		}
		.message.svelte-1nr59td {
			width: 100%;
		}
	}

	.avatar-container.svelte-1nr59td {
		align-self: flex-start;
		position: relative;
		display: flex;
		justify-content: flex-start;
		align-items: flex-start;
		width: 35px;
		height: 35px;
		flex-shrink: 0;
		bottom: 0;
		border-radius: 50%;
		border: 1px solid var(--border-color-primary);
	}
	.user-row.svelte-1nr59td > .avatar-container:where(.svelte-1nr59td) {
		order: 2;
	}

	.user-row.bubble.svelte-1nr59td > .avatar-container:where(.svelte-1nr59td) {
		margin-left: var(--spacing-xxl);
	}

	.bot-row.bubble.svelte-1nr59td > .avatar-container:where(.svelte-1nr59td) {
		margin-right: var(--spacing-xxl);
		margin-left: 0;
	}

	.panel.user-row.svelte-1nr59td > .avatar-container:where(.svelte-1nr59td) {
		order: 0;
	}

	.avatar-container.svelte-1nr59td:not(.thumbnail-item) img {
		width: 100%;
		height: 100%;
		object-fit: cover;
		border-radius: 50%;
		padding: var(--size-1-5);
	}

	.selectable.svelte-1nr59td {
		cursor: pointer;
	}

	@keyframes svelte-1nr59td-dot-flashing {
		0% {
			opacity: 0.8;
		}
		50% {
			opacity: 0.5;
		}
		100% {
			opacity: 0.8;
		}
	}

	/* Image preview */
	.message.svelte-1nr59td .preview {
		object-fit: contain;
		width: 95%;
		max-height: 93%;
	}
	.image-preview.svelte-1nr59td {
		position: absolute;
		z-index: 999;
		left: 0;
		top: 0;
		width: 100%;
		height: 100%;
		overflow: auto;
		background-color: rgba(0, 0, 0, 0.9);
		display: flex;
		justify-content: center;
		align-items: center;
	}
	.image-preview.svelte-1nr59td svg {
		stroke: white;
	}
	.image-preview-close-button.svelte-1nr59td {
		position: absolute;
		top: 10px;
		right: 10px;
		background: none;
		border: none;
		font-size: 1.5em;
		cursor: pointer;
		height: 30px;
		width: 30px;
		padding: 3px;
		background: var(--bg-color);
		box-shadow: var(--shadow-drop);
		border: 1px solid var(--button-secondary-border-color);
		border-radius: var(--radius-lg);
	}

	.message.svelte-1nr59td > div:where(.svelte-1nr59td) {
		width: 100%;
	}
	.html.svelte-1nr59td {
		padding: 0;
		border: none;
		background: none;
	}

	.panel.svelte-1nr59td .bot:where(.svelte-1nr59td),
	.panel.svelte-1nr59td .user:where(.svelte-1nr59td) {
		border: none;
		box-shadow: none;
		background-color: var(--background-fill-secondary);
	}

	textarea.svelte-1nr59td {
		background: none;
		border-radius: var(--radius-lg);
		border: none;
		display: block;
		max-width: 100%;
	}
	.user.svelte-1nr59td textarea:where(.svelte-1nr59td) {
		border-bottom-right-radius: 0;
	}
	.bot.svelte-1nr59td textarea:where(.svelte-1nr59td) {
		border-bottom-left-radius: 0;
	}
	.user.svelte-1nr59td textarea:where(.svelte-1nr59td):focus {
		outline: 2px solid var(--border-color-accent);
	}
	.bot.svelte-1nr59td textarea:where(.svelte-1nr59td):focus {
		outline: 2px solid var(--border-color-primary);
	}

	.panel.user-row.svelte-1nr59td {
		background-color: var(--color-accent-soft);
	}

	.panel.svelte-1nr59td .user-row:where(.svelte-1nr59td),
	.panel.svelte-1nr59td .bot-row:where(.svelte-1nr59td) {
		align-self: flex-start;
	}

	.panel.svelte-1nr59td .user:where(.svelte-1nr59td) *,
	.panel.svelte-1nr59td .bot:where(.svelte-1nr59td) * {
		text-align: inherit;
	}

	.panel.svelte-1nr59td .user:where(.svelte-1nr59td) {
		background-color: var(--color-accent-soft);
	}

	.panel.svelte-1nr59td .user-row:where(.svelte-1nr59td) {
		background-color: var(--color-accent-soft);
		align-self: start;
	}

	.panel.svelte-1nr59td .message:where(.svelte-1nr59td) {
		margin-bottom: var(--spacing-md);
	}
	.container.svelte-stpvyx {
		display: flex;
		margin: calc(var(--spacing-xl) * 2);
	}

	.bubble.pending.svelte-stpvyx {
		border-width: 1px;
		border-radius: var(--radius-lg);
		border-bottom-left-radius: 0;
		border-color: var(--border-color-primary);
		background-color: var(--background-fill-secondary);
		box-shadow: var(--shadow-drop);
		align-self: flex-start;
		width: fit-content;
		margin-bottom: var(--spacing-xl);
	}

	.bubble.with_opposite_avatar.svelte-stpvyx {
		margin-right: calc(var(--spacing-xxl) + 35px + var(--spacing-xxl));
	}

	.panel.pending.svelte-stpvyx {
		margin: 0;
		padding: calc(var(--spacing-lg) * 2) calc(var(--spacing-lg) * 2);
		width: 100%;
		border: none;
		background: none;
		box-shadow: none;
		border-radius: 0;
	}

	.panel.with_avatar.svelte-stpvyx {
		padding-left: calc(var(--spacing-xl) * 2) !important;
		padding-right: calc(var(--spacing-xl) * 2) !important;
	}

	.avatar-container.svelte-stpvyx {
		align-self: flex-start;
		position: relative;
		display: flex;
		justify-content: flex-start;
		align-items: flex-start;
		width: 35px;
		height: 35px;
		flex-shrink: 0;
		bottom: 0;
		border-radius: 50%;
		border: 1px solid var(--border-color-primary);
		margin-right: var(--spacing-xxl);
	}

	.avatar-container.svelte-stpvyx:not(.thumbnail-item) img {
		width: 100%;
		height: 100%;
		object-fit: cover;
		border-radius: 50%;
		padding: var(--size-1-5);
	}

	.message-content.svelte-stpvyx {
		padding: var(--spacing-sm) var(--spacing-xl);
		min-height: var(--size-8);
		display: flex;
		align-items: center;
	}

	.dots.svelte-stpvyx {
		display: flex;
		gap: var(--spacing-xs);
		align-items: center;
	}

	.dot.svelte-stpvyx {
		width: var(--size-1-5);
		height: var(--size-1-5);
		margin-right: var(--spacing-xs);
		border-radius: 50%;
		background-color: var(--body-text-color);
		opacity: 0.5;
		animation: svelte-stpvyx-pulse 1.5s infinite;
	}

	.dot.svelte-stpvyx:nth-child(2) {
		animation-delay: 0.2s;
	}

	.dot.svelte-stpvyx:nth-child(3) {
		animation-delay: 0.4s;
	}

	@keyframes svelte-stpvyx-pulse {
		0%,
		100% {
			opacity: 0.4;
			transform: scale(1);
		}
		50% {
			opacity: 1;
			transform: scale(1.1);
		}
	}
	.placeholder-content.svelte-1rn3hyj {
		display: flex;
		flex-direction: column;
		height: 100%;
	}

	.placeholder.svelte-1rn3hyj {
		align-items: center;
		display: flex;
		justify-content: center;
		height: 100%;
		flex-grow: 1;
	}

	.examples.svelte-1rn3hyj img {
		pointer-events: none;
	}

	.examples.svelte-1rn3hyj {
		margin: auto;
		padding: var(--spacing-xxl);
		display: grid;
		grid-template-columns: repeat(auto-fit, minmax(240px, 1fr));
		gap: var(--spacing-xl);
		max-width: calc(min(4 * 240px + 5 * var(--spacing-xxl), 100%));
	}

	.example.svelte-1rn3hyj {
		display: flex;
		flex-direction: column;
		align-items: flex-start;
		padding: var(--spacing-xxl);
		border: none;
		border-radius: var(--radius-lg);
		background-color: var(--block-background-fill);
		cursor: pointer;
		transition: all 150ms ease-in-out;
		width: 100%;
		gap: var(--spacing-sm);
		border: var(--block-border-width) solid var(--block-border-color);
		transform: translateY(0px);
	}

	.example.svelte-1rn3hyj:hover {
		transform: translateY(-2px);
		background-color: var(--color-accent-soft);
	}

	.example-content.svelte-1rn3hyj {
		display: flex;
		flex-direction: column;
		align-items: flex-start;
		width: 100%;
		height: 100%;
	}

	.example-text-content.svelte-1rn3hyj {
		margin-top: var(--spacing-sm);
		text-align: start;
	}

	.example-text.svelte-1rn3hyj {
		font-size: var(--text-md);
		text-align: start;
		overflow: hidden;
		text-overflow: ellipsis;
	}

	.example-icons-grid.svelte-1rn3hyj {
		display: flex;
		gap: var(--spacing-sm);
		width: 100%;
	}

	.example-icon.svelte-1rn3hyj {
		flex-shrink: 0;
		width: var(--size-8);
		height: var(--size-8);
		display: flex;
		align-items: center;
		justify-content: center;
		border-radius: var(--radius-lg);
		border: var(--block-border-width) solid var(--block-border-color);
		background-color: var(--block-background-fill);
		position: relative;
	}

	.example-icon.svelte-1rn3hyj svg {
		width: var(--size-4);
		height: var(--size-4);
		color: var(--color-text-secondary);
	}

	.text-icon-aa.svelte-1rn3hyj {
		font-size: var(--text-sm);
		font-weight: var(--weight-semibold);
		color: var(--color-text-secondary);
		line-height: 1;
	}

	.example-image-container.svelte-1rn3hyj {
		width: var(--size-8);
		height: var(--size-8);
		border-radius: var(--radius-lg);
		overflow: hidden;
		position: relative;
	}

	.example-image-container.svelte-1rn3hyj img {
		width: 100%;
		height: 100%;
		object-fit: cover;
	}

	.image-overlay.svelte-1rn3hyj {
		position: absolute;
		top: 0;
		left: 0;
		right: 0;
		bottom: 0;
		background: rgba(0, 0, 0, 0.6);
		color: white;
		display: flex;
		align-items: center;
		justify-content: center;
		font-size: var(--text-lg);
		font-weight: var(--weight-semibold);
		border-radius: var(--radius-lg);
	}

	.file-overlay.svelte-1rn3hyj {
		position: absolute;
		inset: 0;
		background: rgba(0, 0, 0, 0.6);
		color: white;
		display: flex;
		align-items: center;
		justify-content: center;
		font-size: var(--text-sm);
		font-weight: var(--weight-semibold);
		border-radius: var(--radius-lg);
	}
	.panel-wrap.svelte-kpz1 {
		width: 100%;
		overflow-y: auto;
	}

	.bubble-wrap.svelte-kpz1 {
		width: 100%;
		overflow-y: auto;
		height: 100%;
		padding-top: var(--spacing-xxl);
	}

	@media (prefers-color-scheme: dark) {
		.bubble-wrap.svelte-kpz1 {
			background: var(--background-fill-secondary);
		}
	}

	.message-wrap.svelte-kpz1 .prose.chatbot.md {
		opacity: 0.8;
		overflow-wrap: break-word;
	}

	.message-wrap.svelte-kpz1 .message-row .md img {
		border-radius: var(--radius-xl);
		margin: var(--size-2);
		width: 400px;
		max-width: 30vw;
		max-height: 30vw;
	}

	/* link styles */
	.message-wrap.svelte-kpz1 .message a {
		color: var(--color-text-link);
		text-decoration: underline;
	}

	/* table styles */
	.message-wrap.svelte-kpz1 .bot:not(:has(.table-wrap)) table,
	.message-wrap.svelte-kpz1 .bot:not(:has(.table-wrap)) tr,
	.message-wrap.svelte-kpz1 .bot:not(:has(.table-wrap)) td,
	.message-wrap.svelte-kpz1 .bot:not(:has(.table-wrap)) th {
		border: 1px solid var(--border-color-primary);
	}

	.message-wrap.svelte-kpz1 .user table,
	.message-wrap.svelte-kpz1 .user tr,
	.message-wrap.svelte-kpz1 .user td,
	.message-wrap.svelte-kpz1 .user th {
		border: 1px solid var(--border-color-accent);
	}

	/* KaTeX */
	.message-wrap.svelte-kpz1 span.katex {
		font-size: var(--text-lg);
		direction: ltr;
	}

	.message-wrap.svelte-kpz1 span.katex-display {
		margin-top: 0;
	}

	.message-wrap.svelte-kpz1 pre {
		position: relative;
	}

	.message-wrap.svelte-kpz1 .grid-wrap {
		max-height: 80% !important;
		max-width: 600px;
		object-fit: contain;
	}

	.message-wrap.svelte-kpz1 > div:where(.svelte-kpz1) p:not(:first-child) {
		margin-top: var(--spacing-xxl);
	}

	.message-wrap.svelte-kpz1 {
		display: flex;
		flex-direction: column;
		justify-content: space-between;
		margin-bottom: var(--spacing-xxl);
		direction: ltr;
	}

	.panel-wrap.svelte-kpz1 .message-row:first-child {
		padding-top: calc(var(--spacing-xxl) * 2);
	}

	.scroll-down-button-container.svelte-kpz1 {
		position: absolute;
		bottom: 10px;
		left: 50%;
		transform: translateX(-50%);
		z-index: var(--layer-top);
	}
	.scroll-down-button-container.svelte-kpz1 button {
		border-radius: 50%;
		box-shadow: var(--shadow-drop);
		transition:
			box-shadow 0.2s ease-in-out,
			transform 0.2s ease-in-out;
	}
	.scroll-down-button-container.svelte-kpz1 button:hover {
		box-shadow:
			var(--shadow-drop),
			0 2px 2px rgba(0, 0, 0, 0.05);
		transform: translateY(-2px);
	}

	.options.svelte-kpz1 {
		margin-left: auto;
		padding: var(--spacing-xxl);
		display: grid;
		grid-template-columns: repeat(auto-fit, minmax(200px, 1fr));
		gap: var(--spacing-xxl);
		max-width: calc(min(4 * 200px + 5 * var(--spacing-xxl), 100%));
		justify-content: flex-end;
	}

	.option.svelte-kpz1 {
		display: flex;
		flex-direction: column;
		align-items: center;
		padding: var(--spacing-xl);
		border: 1px dashed var(--border-color-primary);
		border-radius: var(--radius-md);
		background-color: var(--background-fill-secondary);
		cursor: pointer;
		transition: var(--button-transition);
		max-width: var(--size-56);
		width: 100%;
		justify-content: center;
	}

	.option.svelte-kpz1:hover {
		background-color: var(--color-accent-soft);
		border-color: var(--border-color-accent);
	}
	.wrapper.svelte-1wizwbi {
		display: flex;
		position: relative;
		flex-direction: column;
		align-items: start;
		width: 100%;
		height: 100%;
		flex-grow: 1;
	}

	.progress-text {
		right: auto;
	}