const _name = "Norsk bokmål";
const annotated_image = { "annotated_image": "Annotert bilde" };
const audio = { "allow_recording_access": "Vennligst tillat mikrofontilgang for opptak.", "audio": "Lyd", "record_from_microphone": "Ta opp fra mikrofon", "stop_recording": "Stopp opptak", "no_device_support": "Kan ikke få tilgang til medieenheter. Sørg for at du kjører på en sikker kilde (https) eller localhost (eller har gitt et gyldig SSL-sertifikat til ssl_verify), og at du har gitt nettleseren tilgang til enheten din.", "stop": "Stopp", "resume": "Fortsett", "record": "Ta opp", "no_microphone": "Ingen mikrofon funnet", "pause": "Pause", "play": "Spill av", "waiting": "Venter", "drop_to_upload": "Slipp en lydfil her for å laste opp" };
const blocks = { "connection_can_break": "På mobil kan tilkoblingen brytes hvis denne fanen mister fokus eller enheten går i dvale, og du mister plassen din i køen.", "long_requests_queue": "Det er en lang kø med ventende forespørsler. Dupliser denne Space for å hoppe over køen.", "lost_connection": "Mistet tilkobling på grunn av at siden ble forlatt. Går tilbake til køen...", "waiting_for_inputs": "Venter på at fil(er) skal bli lastet opp, vennligst prøv igjen." };
const checkbox = { "checkbox": "Avkrysningsboks", "checkbox_group": "Avkrysningsboksgruppe" };
const code = { "code": "Kode" };
const color_picker = { "color_picker": "Fargevelger" };
const common = { "built_with": "bygget med", "built_with_gradio": "Bygget med Gradio", "clear": "Tøm", "download": "Last ned", "edit": "Rediger", "empty": "Tom", "error": "Feil", "hosted_on": "Hostet på", "loading": "Laster", "logo": "Logo", "or": "eller", "remove": "Fjern", "settings": "Innstillinger", "share": "Del", "submit": "Send", "undo": "Angre", "no_devices": "Ingen enheter funnet", "language": "Språk", "display_theme": "Visningstema", "pwa": "Progressiv webapplikasjon" };
const dataframe = { "incorrect_format": "Feil format, kun CSV- og TSV-filer støttes", "new_column": "Legg til kolonne", "new_row": "Ny rad", "add_row_above": "Legg til rad over", "add_row_below": "Legg til rad under", "add_column_left": "Legg til kolonne til venstre", "add_column_right": "Legg til kolonne til høyre", "delete_row": "Slett rad", "delete_column": "Slett kolonne", "sort_column": "Sorter kolonne", "sort_ascending": "Sorter stigende", "sort_descending": "Sorter synkende", "drop_to_upload": "Slipp CSV- eller TSV-filer her for å importere data til dataramme", "clear_sort": "Fjern sortering" };
const dropdown = { "dropdown": "Nedtrekksmeny" };
const errors = { "build_error": "Det er en byggefeil", "config_error": "Det er en konfigurasjonsfeil", "contact_page_author": "Vennligst kontakt sidens forfatter for å informere dem.", "no_app_file": "Det er ingen app-fil", "runtime_error": "Det er en kjøretidsfeil", "space_not_working": '"Space fungerer ikke fordi" {0}', "space_paused": "Space er pauset", "use_via_api": "Bruk via API", "use_via_api_or_mcp": "Bruk via API eller MCP" };
const file = { "uploading": "Laster opp..." };
const highlighted_text = { "highlighted_text": "Uthevet tekst" };
const image = { "allow_webcam_access": "Vennligst tillat webkameratilgang for opptak.", "brush_color": "Penselfarge", "brush_radius": "Penselstørrelse", "image": "Bilde", "remove_image": "Fjern bilde", "select_brush_color": "Velg penselfarge", "start_drawing": "Start tegning", "use_brush": "Bruk pensel", "drop_to_upload": "Slipp en bildefil her for å laste opp" };
const label = { "label": "Etikett" };
const login = { "enable_cookies": "Hvis du besøker en HuggingFace Space i inkognitomodus, må du aktivere tredjeparts informasjonskapsler.", "incorrect_credentials": "Feil påloggingsinformasjon", "username": "Brukernavn", "password": "Passord", "login": "Logg inn" };
const number = { "number": "Tall" };
const plot = { "plot": "Plott" };
const radio = { "radio": "Radioknapp" };
const slider = { "slider": "Glidebryter" };
const upload_text = { "click_to_upload": "Klikk for å laste opp", "drop_audio": "Slipp lyd her", "drop_csv": "Slipp CSV her", "drop_file": "Slipp fil her", "drop_image": "Slipp bilde her", "drop_video": "Slipp video her", "drop_gallery": "Slipp media her", "paste_clipboard": "Lim inn fra utklippstavle" };
const video = { "drop_to_upload": "Slipp en videofil her for å laste opp" };
const chatbot = { "edit": "Rediger", "retry": "Prøv igjen", "undo": "Angre", "submit": "Send", "cancel": "Avbryt", "like": "Liker", "dislike": "Liker ikke", "clear": "Tøm samtalen" };
const nb = {
  _name,
  "3D_model": { "3d_model": "3D-modell", "drop_to_upload": "Slipp en 3D-modell (.obj, .glb, .stl, .gltf, .splat eller .ply) fil her for å laste opp" },
  annotated_image,
  audio,
  blocks,
  checkbox,
  code,
  color_picker,
  common,
  dataframe,
  dropdown,
  errors,
  file,
  highlighted_text,
  image,
  label,
  login,
  number,
  plot,
  radio,
  slider,
  upload_text,
  video,
  chatbot
};

export { _name, annotated_image, audio, blocks, chatbot, checkbox, code, color_picker, common, dataframe, nb as default, dropdown, errors, file, highlighted_text, image, label, login, number, plot, radio, slider, upload_text, video };
//# sourceMappingURL=nb-DEqxD3Ip.js.map
