const _name = "العربية";
const annotated_image = {"annotated_image":"صورة مشروحة"};
const audio = {"allow_recording_access":"يرجى السماح بالوصول إلى الميكروفون للتسجيل","audio":"صوتي","record_from_microphone":"تسجيل من الميكروفون","stop_recording":"إيقاف التسجيل","no_device_support":"لا يمكن الوصول إلى أجهزة الوسائط. تأكد من أنك تعمل على مصدر آمن (https) أو localhost (أو قمت بتمرير شهادة SSL صالحة إلى ssl_verify)، وأنك سمحت للمتصفح بالوصول إلى جهازك.","stop":"إيقاف","resume":"استئناف","record":"تسجيل","no_microphone":"لم يتم العثور على ميكروفون","pause":"إيقاف مؤقت","play":"تشغيل","waiting":"جاري الانتظار","drop_to_upload":"اسقط ملف صوت هنا للتحميل"};
const blocks = {"connection_can_break":"على الهاتف المحمول، يمكن أن ينقطع الاتصال إذا تم تغيير التبويب لهذه الصفحةة أو دخل الجهاز في وضع السكون، مما يؤدي إلى فقدان موقعك في قائمة الانتظار.","long_requests_queue":"هناك قائمة انتظار طويلة من الطلبات المعلقة. قم بنسخ هذه المساحة للتخطي.","lost_connection":"تم فقدان الاتصال بسبب مغادرة الصفحة. جارٍ إعادة الانضمام إلى قائمة الانتظار...","waiting_for_inputs":"إنتظر انتهاء رفع الملف(ات)، يرجى إعادة المحاولة."};
const checkbox = {"checkbox":"خانة اختيار","checkbox_group":"مجموعة خانات الاختيار"};
const code = {"code":"الكود"};
const color_picker = {"color_picker":"منتقي الألوان"};
const common = {"built_with":"بُني باستخدام","built_with_gradio":"بني بإستخدام Gradio","clear":"أمسح","download":"تنزيل","edit":"تعديل","empty":"فارغ","error":"خطأ","hosted_on":"مستضاف على","loading":"جاري التحميل","logo":"شعار","or":"أو","remove":"إزالة","settings":"الإعدادات","share":"مشاركة","submit":"أرسل","undo":"تراجع","no_devices":"لم يتم العثور على أجهزة","language":"اللغة","display_theme":"مظهر العرض","pwa":"تطبيق ويب تقدمي","record":"تسجيل","stop_recording":"إيقاف التسجيل","screen_studio":"شاشة الأستوديو","share_gradio_tab":"مشاركة تبويبة صفحة Gradio","run":"تشغيل"};
const dataframe = {"incorrect_format":"تنسيق غير صحيح، يتم دعم ملفات CSV و TSV فقط","new_column":"إضافة عمود","new_row":"صف جديد","add_row_above":"إضافة صف فوق","add_row_below":"إضافة صف تحت","add_column_left":"إضافة عمود لليسار","add_column_right":"إضافة عمود لليمين","delete_row":"حذف الصف","delete_column":"حذف العمود","sort_column":"فرز العمود","sort_ascending":"فرز تصاعدي","sort_descending":"فرز تنازلي","drop_to_upload":"اسقط ملفات CSV أو TSV هنا لاستيراد البيانات إلى الإطر الجدولية","clear_sort":"مسح الترتيب","filter":"مرشح","clear_filter":"مسح المرشحات"};
const dropdown = {"dropdown":"قائمة منسدلة"};
const errors = {"build_error":"هناك خطأ في البناء","config_error":"هناك خطأ في التكوين","contact_page_author":"يرجى التواصل مع مبرمج الصفحة","no_app_file":"لا يوجد ملف تطبيق","runtime_error":"هناك خطأ أثناء التشغيل","space_not_working":"المساحة لا تعمل لأن {0}","space_paused":"المساحة متوقفة مؤقتًا","use_via_api":"إستخدام عبر API","use_via_api_or_mcp":"إستخدام عبر واجهة برمجة التطبيقات API أو خادم MCP"};
const file = {"uploading":"جاري الرفع..."};
const highlighted_text = {"highlighted_text":"نص محدد"};
const image = {"allow_webcam_access":"يرجى السماح بالوصول إلى كاميرا الويب للتسجيل.","brush_color":"لون الفرشاة","brush_radius":"حجم الفرشاة","image":"صورة","remove_image":"إزالة الصورة","select_brush_color":"اختر لون الفرشاة","start_drawing":"ابدأ الرسم","use_brush":"استخدم الفرشاة","drop_to_upload":"اسقط ملف الصورة هنا لتحميله"};
const label = {"label":"ملصق"};
const login = {"enable_cookies":"إذا كنت تزور مساحة HuggingFace في وضع التصفح المتخفي، يجب عليك تمكين ملفات تعريف الارتباط من الجهات الخارجية.","incorrect_credentials":"بيانات الاعتماد غير صحيحة","username":"اسم المستخدم","password":"كلمة المرور","login":"تسجيل الدخول"};
const number = {"number":"رقم"};
const plot = {"plot":"رسم بياني"};
const radio = {"radio":"زر راديو"};
const slider = {"slider":"شريط التمرير"};
const upload_text = {"click_to_upload":"انقر للتحميل","drop_audio":"أسقط الملف الصوتي هنا","drop_csv":"أسقط ملف البيانات هنا","drop_file":"أسقط الملف هنا","drop_image":"أسقط الصورة هنا","drop_video":"أسقط الفيديو هنا","drop_gallery":"أسقط الوسائط هنا","paste_clipboard":"لصق من الحافظة"};
const video = {"drop_to_upload":"اسقط ملف الفيديو هنا للتحميل"};
const chatbot = {"edit":"تعديل","retry":"إعادة المحاولة","undo":"تراجع","submit":"إرسال","cancel":"إلغاء","like":"إعجاب","dislike":"عدم الإعجاب","clear":"مسح المحادثة"};
const ar = {
  _name,
  "3D_model": {"3d_model":"نموذج ثلاثي الأبعاد","drop_to_upload":"اسقط ملف نموذج ثلاثي الأبعاد 3D (.obj، .glb، .stl، .gltf، .splat، أو .ply) هنا للتحميل"},
  annotated_image,
  audio,
  blocks,
  checkbox,
  code,
  color_picker,
  common,
  dataframe,
  dropdown,
  errors,
  file,
  highlighted_text,
  image,
  label,
  login,
  number,
  plot,
  radio,
  slider,
  upload_text,
  video,
  chatbot,
};

export { _name, annotated_image, audio, blocks, chatbot, checkbox, code, color_picker, common, dataframe, ar as default, dropdown, errors, file, highlighted_text, image, label, login, number, plot, radio, slider, upload_text, video };
//# sourceMappingURL=ar-Bnr8rik1.js.map
