
	.nav-holder.svelte-99kmwu {
		padding: var(--size-2) 0;
		border-bottom: solid 1px var(--border-color-primary);
	}
	nav.svelte-99kmwu {
		display: flex;
		flex-wrap: wrap;
		gap: var(--size-2);
		justify-content: flex-end;
		margin: 0 auto;
		padding: 0 var(--size-8);
	}
	nav.svelte-99kmwu a:where(.svelte-99kmwu) {
		padding: var(--size-1) var(--size-2);
		border-radius: var(--block-radius);
		border-width: var(--block-border-width);
		border-color: transparent;
		color: var(--body-text-color-subdued);
	}
	nav.svelte-99kmwu a.active:where(.svelte-99kmwu) {
		color: var(--body-text-color);
		border-color: var(--block-border-color);
		background-color: var(--block-background-fill);
	}
	.gradio-container-6-11-0 {
		display: flex;
		position: relative;
		flex-direction: column;
		padding: 0;
		min-height: 1px;
		overflow: hidden;
		color: var(--button-secondary-text-color);
	}

	.embed-container.svelte-99kmwu {
		margin: var(--size-4) 0px;
		border: 1px solid var(--button-secondary-border-color);
		border-radius: var(--embed-radius);
	}

	.with-info.svelte-99kmwu {
		padding-bottom: var(--size-7);
	}

	.embed-container.svelte-99kmwu > .main:where(.svelte-99kmwu) {
		padding: var(--size-4);
	}

	.main.svelte-99kmwu {
		display: flex;
		flex-grow: 1;
		flex-direction: column;
	}

	.app.svelte-99kmwu {
		position: relative;
		margin: auto;
		padding: var(--size-4) var(--size-8);
		width: 100%;
		height: 100%;
	}

	@media (min-width: 640px) {
		.fillable.svelte-99kmwu:not(.fill_width) {
			max-width: 640px;
		}
	}
	@media (min-width: 768px) {
		.fillable.svelte-99kmwu:not(.fill_width) {
			max-width: 768px;
		}
	}
	@media (min-width: 1024px) {
		.fillable.svelte-99kmwu:not(.fill_width) {
			max-width: 1024px;
		}
	}
	@media (min-width: 1280px) {
		.fillable.svelte-99kmwu:not(.fill_width) {
			max-width: 1280px;
		}
	}
	@media (min-width: 1536px) {
		.fillable.svelte-99kmwu:not(.fill_width) {
			max-width: 1536px;
		}
	}
	@media (min-width: 1920px) {
		.fillable.svelte-99kmwu:not(.fill_width) {
			max-width: 1920px;
		}
	}

	.info.svelte-99kmwu {
		display: flex;
		position: absolute;
		bottom: 0;
		justify-content: flex-start;
		border-top: 1px solid var(--button-secondary-border-color);
		padding: var(--size-1) var(--size-5);
		width: 100%;
		color: var(--body-text-color-subdued);
		font-size: var(--text-md);
		white-space: nowrap;
	}

	.info.svelte-99kmwu > span:where(.svelte-99kmwu) {
		word-wrap: break-word;
		-break: keep-all;
		display: block;
		word-break: keep-all;
	}

	.info.svelte-99kmwu > span:where(.svelte-99kmwu):nth-child(1) {
		margin-right: 4px;
		min-width: 0px;
		max-width: max-content;
		overflow: hidden;
		color: var(--body-text-color);
		text-overflow: ellipsis;
		white-space: nowrap;
	}

	.info.svelte-99kmwu > span:where(.svelte-99kmwu):nth-child(2) {
		margin-right: 3px;
	}

	.info.svelte-99kmwu > span:where(.svelte-99kmwu):nth-child(2),
	.info.svelte-99kmwu > span:where(.svelte-99kmwu):nth-child(3) {
		width: max-content;
	}

	.info.svelte-99kmwu > span:where(.svelte-99kmwu):nth-child(3) {
		align-self: flex-end;
		justify-self: flex-end;
		margin-left: auto;
		text-align: right;
	}

	.info.svelte-99kmwu > span:where(.svelte-99kmwu):nth-child(1) {
		flex-shrink: 9;
	}

	.hidden-title.svelte-99kmwu {
		position: absolute;
		left: var(--size-5);
		opacity: 0;
		background: var(--button-secondary-background-fill);
		padding-right: 4px;
	}

	.info.svelte-99kmwu a:where(.svelte-99kmwu) {
		color: var(--body-text-color);
	}

	.title.svelte-99kmwu {
		font-size: var(--text-sm);
		font-family: var(--font-mono);
	}

	.hf.svelte-99kmwu {
		margin-left: 5px;
	}

	.space-logo.svelte-99kmwu img:where(.svelte-99kmwu) {
		display: inline-block;
		margin-bottom: 4px;
		height: 12px;
	}

	.main.svelte-99kmwu a:where(.svelte-99kmwu):hover {
		text-decoration: underline;
	}