
	.mic-select.svelte-ym1wxn {
		height: var(--size-8);
		background: var(--block-background-fill);
		padding: 0px var(--spacing-xxl);
		border-radius: var(--button-large-radius);
		font-size: var(--text-md);
		border: 1px solid var(--block-border-color);
		gap: var(--size-1);
	}

	select.svelte-ym1wxn {
		text-overflow: ellipsis;
		max-width: var(--size-40);
	}

	@media (max-width: 375px) {
		select.svelte-ym1wxn {
			width: 100%;
		}
	}
	.controls.svelte-1xuh0j1 {
		display: flex;
		align-items: center;
		justify-content: space-between;
		flex-wrap: wrap;
	}

	.wrapper.svelte-1xuh0j1 {
		display: flex;
		align-items: center;
		flex-wrap: wrap;
	}

	.record.svelte-1xuh0j1 {
		margin-right: var(--spacing-md);
	}

	.stop-button-paused.svelte-1xuh0j1 {
		display: none;
		height: var(--size-8);
		width: var(--size-20);
		background-color: var(--block-background-fill);
		border-radius: var(--button-large-radius);
		align-items: center;
		border: 1px solid var(--block-border-color);
		margin: var(--size-1) var(--size-1) 0 0;
	}

	.stop-button-paused.svelte-1xuh0j1::before {
		content: "";
		height: var(--size-4);
		width: var(--size-4);
		border-radius: var(--radius-full);
		background: var(--primary-600);
		margin: 0 var(--spacing-xl);
	}
	.stop-button.svelte-1xuh0j1::before {
		content: "";
		height: var(--size-4);
		width: var(--size-4);
		border-radius: var(--radius-full);
		background: var(--primary-600);
		margin: 0 var(--spacing-xl);
		animation: svelte-1xuh0j1-scaling 1800ms infinite;
	}

	.stop-button.svelte-1xuh0j1 {
		display: none;
		height: var(--size-8);
		width: var(--size-20);
		background-color: var(--block-background-fill);
		border-radius: var(--button-large-radius);
		align-items: center;
		border: 1px solid var(--primary-600);
		margin: var(--size-1) var(--size-1) 0 0;
	}

	.record-button.svelte-1xuh0j1::before {
		content: "";
		height: var(--size-4);
		width: var(--size-4);
		border-radius: var(--radius-full);
		background: var(--primary-600);
		margin: 0 var(--spacing-xl);
	}

	.record-button.svelte-1xuh0j1 {
		height: var(--size-8);
		background-color: var(--block-background-fill);
		border-radius: var(--button-large-radius);
		display: flex;
		align-items: center;
		border: 1px solid var(--block-border-color);
		padding-right: var(--spacing-xl);
	}

	.duration-button.svelte-1xuh0j1 {
		border-radius: var(--button-large-radius);
	}

	.stop-button.svelte-1xuh0j1:disabled {
		cursor: not-allowed;
	}

	.record-button.svelte-1xuh0j1:disabled {
		cursor: not-allowed;
		opacity: 0.5;
	}

	@keyframes svelte-1xuh0j1-scaling {
		0% {
			background-color: var(--primary-600);
			scale: 1;
		}
		50% {
			background-color: var(--primary-600);
			scale: 1.2;
		}
		100% {
			background-color: var(--primary-600);
			scale: 1;
		}
	}

	.pause-button.svelte-1xuh0j1 {
		display: none;
		height: var(--size-8);
		width: var(--size-20);
		border: 1px solid var(--block-border-color);
		border-radius: var(--button-large-radius);
		padding: var(--spacing-md);
		margin: var(--size-1) var(--size-1) 0 0;
	}

	.resume-button.svelte-1xuh0j1 {
		display: none;
		height: var(--size-8);
		width: var(--size-20);
		border: 1px solid var(--block-border-color);
		border-radius: var(--button-large-radius);
		padding: var(--spacing-xl);
		line-height: 1px;
		font-size: var(--text-md);
		margin: var(--size-1) var(--size-1) 0 0;
	}

	.duration.svelte-1xuh0j1 {
		display: flex;
		height: var(--size-8);
		width: var(--size-20);
		border: 1px solid var(--block-border-color);
		padding: var(--spacing-md);
		align-items: center;
		justify-content: center;
		margin: var(--size-1) var(--size-1) 0 0;
	}

	::part(region) {
		border-radius: var(--radius-md);
		height: 98% !important;
		border: 1px solid var(--trim-region-color);
		background-color: unset;
		border-width: 1px 3px;
	}

	::part(region)::after {
		content: "";
		position: absolute;
		top: 0;
		left: 0;
		width: 100%;
		height: 100%;
		background: var(--trim-region-color);
		opacity: 0.2;
		border-radius: var(--radius-md);
	}

	::part(region-handle) {
		width: 5px !important;
		border: none;
	}
	.microphone.svelte-j9q3sk {
		width: 100%;
		display: none;
	}

	.component-wrapper.svelte-j9q3sk {
		padding: var(--size-3);
		width: 100%;
	}

	.timestamps.svelte-j9q3sk {
		display: flex;
		justify-content: space-between;
		align-items: center;
		width: 100%;
		padding: var(--size-1) 0;
		margin: var(--spacing-md) 0;
	}

	.time.svelte-j9q3sk {
		color: var(--neutral-400);
	}

	.duration.svelte-j9q3sk {
		color: var(--neutral-400);
	}

	.trim-duration.svelte-j9q3sk {
		color: var(--color-accent);
		margin-right: var(--spacing-sm);
	}
	.controls.svelte-m6ymia {
		display: flex;
		align-items: center;
		justify-content: space-between;
		flex-wrap: wrap;
	}

	.mic-wrap.svelte-m6ymia {
		display: block;
		align-items: center;
		margin: var(--spacing-xl);
	}

	.icon.svelte-m6ymia {
		width: var(--size-4);
		height: var(--size-4);
		fill: var(--primary-600);
		stroke: var(--primary-600);
	}

	.stop-button-paused.svelte-m6ymia {
		display: none;
		height: var(--size-8);
		width: var(--size-20);
		background-color: var(--block-background-fill);
		border-radius: var(--button-large-radius);
		align-items: center;
		border: 1px solid var(--block-border-color);
		margin-right: 5px;
	}

	.stop-button-paused.svelte-m6ymia::before {
		content: "";
		height: var(--size-4);
		width: var(--size-4);
		border-radius: var(--radius-full);
		background: var(--primary-600);
		margin: 0 var(--spacing-xl);
	}

	.stop-button.svelte-m6ymia::before {
		content: "";
		height: var(--size-4);
		width: var(--size-4);
		border-radius: var(--radius-full);
		background: var(--primary-600);
		margin: 0 var(--spacing-xl);
		animation: svelte-m6ymia-scaling 1800ms infinite;
	}

	.stop-button.svelte-m6ymia {
		height: var(--size-8);
		width: var(--size-20);
		background-color: var(--block-background-fill);
		border-radius: var(--button-large-radius);
		align-items: center;
		border: 1px solid var(--primary-600);
		margin-right: 5px;
		display: flex;
	}

	.spinner-button.svelte-m6ymia {
		height: var(--size-8);
		width: var(--size-24);
		background-color: var(--block-background-fill);
		border-radius: var(--radius-3xl);
		align-items: center;
		border: 1px solid var(--primary-600);
		margin: 0 var(--spacing-xl);
		display: flex;
		justify-content: space-evenly;
	}

	.record-button.svelte-m6ymia::before {
		content: "";
		height: var(--size-4);
		width: var(--size-4);
		border-radius: var(--radius-full);
		background: var(--primary-600);
		margin: 0 var(--spacing-xl);
	}

	.record-button.svelte-m6ymia {
		height: var(--size-8);
		width: var(--size-24);
		background-color: var(--block-background-fill);
		border-radius: var(--button-large-radius);
		display: flex;
		align-items: center;
		border: 1px solid var(--block-border-color);
	}

	@keyframes svelte-m6ymia-scaling {
		0% {
			background-color: var(--primary-600);
			scale: 1;
		}
		50% {
			background-color: var(--primary-600);
			scale: 1.2;
		}
		100% {
			background-color: var(--primary-600);
			scale: 1;
		}
	}
	.audio-container.svelte-ocxd3m {
		height: calc(var(--size-full) - var(--size-6));
		display: flex;
		flex-direction: column;
		justify-content: space-between;
	}