const _name = "Українська";
const common = { "built_with_gradio": "Зроблено на основі Gradio", "clear": "Очистити", "or": "або", "submit": "Надіслати", "settings": "Налаштування", "built_with": "Зроблено з", "download": "Завантажити", "edit": "Редагувати", "empty": "Порожній", "error": "Помилка", "hosted_on": "Розміщено на", "loading": "Завантаження", "logo": "Логотип", "remove": "Видалити", "share": "Поділитися", "undo": "Скасувати", "no_devices": "Пристроїв не знайдено", "language": "Мова", "display_theme": "Тема оформлення", "pwa": "Прогресивний веб-додаток" };
const upload_text = { "click_to_upload": "Натисніть, щоб завантажити", "drop_audio": "Перетягніть аудіо сюди", "drop_csv": "Перетягніть CSV-файл сюди", "drop_file": "Перетягніть файл сюди", "drop_image": "Перетягніть зображення сюди", "drop_video": "Перетягніть відео сюди", "drop_gallery": "Перетягніть медіа сюди", "paste_clipboard": "Вставити з буфера обміну" };
const annotated_image = { "annotated_image": "Анотоване зображення" };
const audio = { "allow_recording_access": "Будь ласка, дайте доступ до мікрофона для запису.", "audio": "Аудіо", "drop_to_upload": "Перетягніть аудіофайл сюди, щоб завантажити", "record_from_microphone": "Записати з мікрофона", "stop_recording": "Зупинити запис", "no_device_support": "Медійні пристрої не можуть бути доступні. Переконайтеся, що ви запускаєте з захищеного джерела (https) або localhost (або ви передали дійсний SSL-сертифікат до ssl_verify), і ви дозволили браузеру доступ до вашого пристрою.", "stop": "Зупинити", "resume": "Відновити", "record": "Записати", "no_microphone": "Мікрофон не виявлено", "pause": "Пауза", "play": "Відтворити", "waiting": "Очікування" };
const blocks = { "connection_can_break": "На мобільному пристрої з'єднання може перерватися, якщо ця вкладка втрачає фокус або пристрій увійде в режим сну, в результаті чого ви втратите своє місце в черзі.", "long_requests_queue": "У черзі залишилось багато запитів. Скопіюйте цей Space, щоб пропустити чергу.", "lost_connection": "З'єднання втрачено через вихід зі сторінки. Переміщення до кінця черги...", "waiting_for_inputs": "Чекаємо завершення завантаження файлу(ів), будь ласка, спробуйте ще раз." };
const checkbox = { "checkbox": "Прапорець", "checkbox_group": "Група прапорців" };
const code = { "code": "Код" };
const color_picker = { "color_picker": "Вибір кольору" };
const dataframe = { "incorrect_format": "Неправильний формат, підтримуються лише файли CSV та TSV", "new_column": "Додати стовпець", "new_row": "Новий рядок", "add_row_above": "Додати рядок вище", "add_row_below": "Додати рядок нижче", "delete_row": "Видалити рядок", "delete_column": "Видалити стовпець", "add_column_left": "Додати стовпець зліва", "add_column_right": "Додати стовпець справа", "sort_column": "Сортувати стовпець", "sort_ascending": "Сортувати за зростанням", "sort_descending": "Сортувати за спаданням", "drop_to_upload": "Перетягніть CSV або TSV файли сюди, щоб імпортувати дані в таблицю даних", "clear_sort": "Очистити сортування" };
const dropdown = { "dropdown": "Випадне меню" };
const errors = { "build_error": "Є помилка побудови", "config_error": "Є помилка конфігурації", "contact_page_author": "Будь ласка, зв'яжіться з автором сторінки, щоб повідомити.", "no_app_file": "Немає файлу додатку", "runtime_error": "Є помилка виконання", "space_not_working": '"Space не працює, оскільки" {0}', "space_paused": "Space призупинено", "use_via_api": "Використовувати через API", "use_via_api_or_mcp": "Використовувати через API або MCP" };
const file = { "uploading": "Завантаження..." };
const highlighted_text = { "highlighted_text": "Виділений текст" };
const image = { "allow_webcam_access": "Будь ласка, дайте доступ до веб-камери для запису.", "brush_color": "Колір пензля", "brush_radius": "Розмір пензля", "image": "Зображення", "remove_image": "Видалити зображення", "select_brush_color": "Виберіть колір пензля", "start_drawing": "Почніть малювати", "use_brush": "Використовуйте пензель", "drop_to_upload": "Перетягніть файл зображення сюди, щоб завантажити" };
const label = { "label": "Мітка" };
const login = { "enable_cookies": "Якщо ви відвідуєте HuggingFace Space у режимі Інкогніто, вам потрібно увімкнути сторонні куки.", "incorrect_credentials": "Неправильні облікові дані", "username": "Ім'я користувача", "password": "Пароль", "login": "Увійти" };
const number = { "number": "Число" };
const plot = { "plot": "Діаграма" };
const radio = { "radio": "Перемикач" };
const slider = { "slider": "Повзунок" };
const video = { "drop_to_upload": "Перетягніть відеофайл сюди, щоб завантажити" };
const chatbot = { "edit": "Редагувати", "retry": "Повторити", "undo": "Скасувати", "submit": "Надіслати", "cancel": "Скасувати", "like": "Подобається", "dislike": "Не подобається", "clear": "Очистити чат" };
const uk = {
  _name,
  common,
  upload_text,
  "3D_model": { "3d_model": "3D-модель", "drop_to_upload": "Перетягніть файл 3D-моделі (.obj, .glb, .stl, .gltf, .splat або .ply) сюди, щоб завантажити" },
  annotated_image,
  audio,
  blocks,
  checkbox,
  code,
  color_picker,
  dataframe,
  dropdown,
  errors,
  file,
  highlighted_text,
  image,
  label,
  login,
  number,
  plot,
  radio,
  slider,
  video,
  chatbot
};

export { _name, annotated_image, audio, blocks, chatbot, checkbox, code, color_picker, common, dataframe, uk as default, dropdown, errors, file, highlighted_text, image, label, login, number, plot, radio, slider, upload_text, video };
//# sourceMappingURL=uk-CKgyQx4N.js.map
