import './i18n-CMo5lpzy.js';
import { a as append, f as from_svg } from './index-BvBk1Iap.js';

var root = from_svg(`<svg xmlns="http://www.w3.org/2000/svg" width="100%" height="100%" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="1.5" stroke-linecap="round" stroke-linejoin="round" class="feather feather-music"><path d="M9 18V5l12-2v13"></path><circle cx="6" cy="18" r="3"></circle><circle cx="18" cy="16" r="3"></circle></svg>`);

function Music($$anchor) {
	var svg = root();

	append($$anchor, svg);
}

export { Music as M };
//# sourceMappingURL=Music-D0NN2hV9.js.map
