
	.wrap.svelte-8epfm4 {
		display: flex;
		flex-direction: column;
		width: 100%;
	}

	.head.svelte-8epfm4 {
		margin-bottom: var(--size-2);
		display: flex;
		justify-content: space-between;
		align-items: flex-start;
		flex-wrap: wrap;
		width: 100%;
	}

	.head.svelte-8epfm4 > label:where(.svelte-8epfm4) {
		flex: 1;
	}

	.head.svelte-8epfm4 > .tab-like-container:where(.svelte-8epfm4) {
		margin-left: auto;
		order: 1;
	}

	.slider_input_container.svelte-8epfm4 {
		display: flex;
		align-items: center;
		gap: var(--size-2);
	}

	input[type="range"].svelte-8epfm4 {
		-webkit-appearance: none;
		appearance: none;
		width: 100%;
		cursor: pointer;
		outline: none;
		border-radius: var(--radius-xl);
		min-width: var(--size-28);
		background: transparent;
	}

	/* webkit track */
	input[type="range"].svelte-8epfm4::-webkit-slider-runnable-track {
		height: var(--size-2);
		background: var(--neutral-200);
		border-radius: var(--radius-xl);
	}

	/* webkit thumb */
	input[type="range"].svelte-8epfm4::-webkit-slider-thumb {
		-webkit-appearance: none;
		appearance: none;
		height: var(--size-4);
		width: var(--size-4);
		background-color: white;
		border-radius: 50%;
		margin-top: -5px;
		box-shadow:
			0 0 0 1px rgba(247, 246, 246, 0.739),
			1px 1px 4px rgba(0, 0, 0, 0.1);
	}

	input[type="range"].svelte-8epfm4::-webkit-slider-runnable-track {
		background: linear-gradient(
			to right,
			var(--slider-color) var(--range_progress),
			var(--neutral-200) var(--range_progress)
		);
	}

	/* firefox */
	input[type="range"].svelte-8epfm4::-moz-range-track {
		height: var(--size-2);
		background: var(--neutral-200);
		border-radius: var(--radius-xl);
	}

	input[type="range"].svelte-8epfm4::-moz-range-thumb {
		appearance: none;
		height: var(--size-4);
		width: var(--size-4);
		background-color: white;
		border-radius: 50%;
		border: none;
		margin-top: calc(-1 * (var(--size-4) - var(--size-2)) / 2);
		box-shadow:
			0 0 0 1px rgba(247, 246, 246, 0.739),
			1px 1px 4px rgba(0, 0, 0, 0.1);
	}

	input[type="range"].svelte-8epfm4::-moz-range-progress {
		height: var(--size-2);
		background-color: var(--slider-color);
		border-radius: var(--radius-xl);
	}

	input[type="number"].svelte-8epfm4 {
		display: block;
		outline: none;
		border: 1px solid var(--input-border-color);
		border-radius: var(--radius-sm);
		background: var(--input-background-fill);
		padding: var(--size-2) var(--size-3);
		height: var(--size-8);
		color: var(--body-text-color);
		font-size: var(--input-text-size);
		line-height: var(--line-sm);
		text-align: center;
		min-width: var(--size-16);
		transition: border-color 0.15s ease-in-out;
	}

	input[type="number"].svelte-8epfm4:focus {
		box-shadow: none;
		border-width: 2px;
	}

	input.svelte-8epfm4:disabled,
	input[disabled].svelte-8epfm4 {
		-webkit-text-fill-color: var(--body-text-color);
		opacity: 1;
		cursor: not-allowed;
	}

	input.svelte-8epfm4::placeholder {
		color: var(--input-placeholder-color);
	}

	input[type="range"][disabled].svelte-8epfm4 {
		opacity: 0.6;
	}

	input[type="range"][disabled].svelte-8epfm4::-webkit-slider-thumb,
	input[type="range"][disabled].svelte-8epfm4::-moz-range-thumb,
	input[type="range"][disabled].svelte-8epfm4::-ms-thumb,
	input[type="range"][disabled].svelte-8epfm4::-webkit-slider-thumb:hover,
	input[type="range"][disabled].svelte-8epfm4::-moz-range-thumb:hover,
	input[type="range"][disabled].svelte-8epfm4::-moz-range-track:hover {
		background-color: var(--body-text-color-subdued);
		cursor: not-allowed;
	}

	.min_value.svelte-8epfm4,
	.max_value.svelte-8epfm4 {
		font-size: var(--text-sm);
		color: var(--body-text-color-subdued);
	}

	.min_value.svelte-8epfm4 {
		margin-right: var(--size-0-5);
	}

	.max_value.svelte-8epfm4 {
		margin-left: var(--size-0-5);
		margin-right: var(--size-0-5);
	}

	@media (max-width: 480px) {
		.min_value.svelte-8epfm4,
		.max_value.svelte-8epfm4 {
			display: none;
		}
	}

	@media (max-width: 520px) {
		.head.svelte-8epfm4 {
			gap: var(--size-3);
		}
	}

	@media (max-width: 420px) {
		.head.svelte-8epfm4 .tab-like-container:where(.svelte-8epfm4) {
			margin-bottom: var(--size-4);
		}
	}

	.tab-like-container.svelte-8epfm4 {
		display: flex;
		align-items: stretch;
		border: 1px solid var(--input-border-color);
		border-radius: var(--radius-sm);
		overflow: hidden;
		height: var(--size-6);
	}

	input[type="number"].svelte-8epfm4 {
		border: none;
		border-radius: 0;
		padding: var(--size-1) var(--size-2);
		height: 100%;
		min-width: var(--size-14);
		font-size: var(--text-sm);
	}

	input[type="number"].svelte-8epfm4:focus {
		box-shadow: inset 0 0 0 1px var(--color-accent);
		border-radius: 3px 0 0px 3px;
	}

	.reset-button.svelte-8epfm4 {
		display: flex;
		align-items: center;
		justify-content: center;
		background: none;
		border: none;
		border-left: 1px solid var(--input-border-color);
		cursor: pointer;
		font-size: var(--text-sm);
		color: var(--body-text-color);
		padding: 0 var(--size-2);
		min-width: var(--size-6);
		transition: background-color 0.15s ease-in-out;
	}

	.reset-button.svelte-8epfm4:hover:not(:disabled) {
		background-color: var(--background-fill-secondary);
	}

	.reset-button.svelte-8epfm4:disabled {
		opacity: 0.5;
		cursor: not-allowed;
	}