
	span.svelte-e5lyqv {
		font-weight: var(--section-header-text-weight);
		font-size: var(--section-header-text-size);
	}
	.label-wrap.svelte-e5lyqv {
		display: flex;
		justify-content: space-between;
		cursor: pointer;
		width: var(--size-full);
		color: var(--accordion-text-color);
	}
	.label-wrap.open.svelte-e5lyqv {
		margin-bottom: var(--size-2);
	}

	.icon.svelte-e5lyqv {
		transition: 150ms;
	}