const _name = "தமிழ்";
const common = {"built_with_gradio":"கிரேடியோவுடன் உருவாக்கப்பட்டது","clear":"அழிக்கவும்","or":"அல்லது","submit":"சமர்ப்பிக்கவும்","built_with":"உருவாக்கப்பட்டது","download":"பதிவிறக்கம்","edit":"திருத்து","empty":"காலியாக உள்ளது","error":"பிழை","hosted_on":"இதில் ஹோஸ்ட் செய்யப்பட்டது","loading":"ஏற்றுகிறது","logo":"லோகோ","remove":"நீக்கு","settings":"அமைப்புகள்","share":"பகிர்","undo":"செயல்தவிர்","no_devices":"சாதனங்கள் எதுவும் கிடைக்கவில்லை","language":"மொழி","display_theme":"காட்சி தீம்","pwa":"முன்னேற்ற வலை பயன்பாடு"};
const upload_text = {"click_to_upload":"பதிவேற்ற கிளிக் செய்யவும்","drop_audio":"ஆடியோவை இங்கே விடவும்","drop_csv":"CSV ஐ இங்கே விடவும்","drop_file":"கோப்பை இங்கே விடவும்","drop_image":"படத்தை இங்கே விடவும்","drop_video":"வீடியோவை இங்கே விடவும்","drop_gallery":"மீடியாவை இங்கே விடவும்","paste_clipboard":"கிளிப்போர்டிலிருந்து ஒட்டவும்"};
const annotated_image = {"annotated_image":"குறிப்பிடப்பட்ட படம்"};
const audio = {"allow_recording_access":"தயவுசெய்து பதிவு செய்ய மைக்ரோஃபோன் அணுகலை அனுமதிக்கவும்.","audio":"ஒலி","drop_to_upload":"ஒலிக் கோப்பை பதிவேற்ற இங்கே விடவும்","record_from_microphone":"மைக்ரோஃபோனிலிருந்து பதிவு செய்யவும்","stop_recording":"பதிவை நிறுத்தவும்","no_device_support":"மீடியா சாதனங்களை அணுக முடியவில்லை. நீங்கள் பாதுகாப்பான மூலத்தில் (https) அல்லது localhost இல் இயங்குகிறீர்களா (அல்லது ssl_verify க்கு செல்லுபடியாகும் SSL சான்றிதழை அனுப்பியுள்ளீர்களா) என்பதையும், உங்கள் சாதனத்திற்கான உலாவி அணுகலை அனுமதித்துள்ளீர்களா என்பதையும் சரிபார்க்கவும்.","stop":"நிறுத்து","resume":"தொடரவும்","record":"பதிவு செய்","no_microphone":"மைக்ரோஃபோன் எதுவும் கிடைக்கவில்லை","pause":"இடைநிறுத்து","play":"இயக்கு","waiting":"காத்திருக்கிறது"};
const blocks = {"connection_can_break":"மொபைலில், இந்தத் தாவல் ஃபோகஸை இழந்தால் அல்லது சாதனம் உறங்கினால், இணைப்பு துண்டிக்கப்பட்டு, வரிசையில் உங்கள் இடத்தை இழக்கலாம்.","long_requests_queue":"காத்திருக்கும் கோரிக்கைகளின் நீண்ட வரிசை உள்ளது. வரிசையைத் தவிர்க்க இந்த Space ஐ நகலெடுக்கவும்.","lost_connection":"பக்கத்தை விட்டு வெளியேறியதால் இணைப்பு துண்டிக்கப்பட்டது. வரிசைக்கு மீண்டும் இணைகிறது...","waiting_for_inputs":"கோப்பு(கள்) பதிவேற்றம் முடிவடைய காத்திருக்கிறது, தயவுசெய்து மீண்டும் முயற்சிக்கவும்."};
const checkbox = {"checkbox":"தேர்வுப்பெட்டி","checkbox_group":"தேர்வுப்பெட்டி குழு"};
const code = {"code":"குறியீடு"};
const color_picker = {"color_picker":"வண்ணத் தேர்வி"};
const dataframe = {"incorrect_format":"தவறான வடிவம், CSV மற்றும் TSV கோப்புகள் மட்டுமே ஆதரிக்கப்படுகின்றன","new_column":"நெடுவரிசை சேர்க்க","new_row":"புதிய வரிசை","add_row_above":"மேலே வரிசை சேர்க்க","add_row_below":"கீழே வரிசை சேர்க்க","delete_row":"வரிசையை நீக்கு","delete_column":"நெடுவரிசையை நீக்கு","add_column_left":"இடது பக்கத்தில் நெடுவரிசை சேர்க்க","add_column_right":"வலது பக்கத்தில் நெடுவரிசை சேர்க்க","sort_column":"நெடுவரிசையை வரிசைப்படுத்து","sort_ascending":"ஏறுவரிசையில் வரிசைப்படுத்து","sort_descending":"இறங்குவரிசையில் வரிசைப்படுத்து","drop_to_upload":"தரவுத்தளத்திற்கு தரவை இறக்குமதி செய்ய CSV அல்லது TSV கோப்புகளை இங்கே விடவும்","clear_sort":"வரிசைப்படுத்தலை அழி"};
const dropdown = {"dropdown":"கீழ்தோன்றல் பட்டியல்"};
const errors = {"build_error":"உருவாக்க பிழை உள்ளது","config_error":"கட்டமைப்பு பிழை உள்ளது","contact_page_author":"தயவுசெய்து பக்க ஆசிரியரைத் தொடர்பு கொண்டு தெரிவிக்கவும்.","no_app_file":"பயன்பாட்டுக் கோப்பு இல்லை","runtime_error":"இயக்க நேர பிழை உள்ளது","space_not_working":"\"Space செயல்படவில்லை ஏனெனில்\" {0}","space_paused":"Space இடைநிறுத்தப்பட்டுள்ளது","use_via_api":"API மூலம் பயன்படுத்தவும்","use_via_api_or_mcp":"API அல்லது MCP மூலம் பயன்படுத்தவும்"};
const file = {"uploading":"பதிவேற்றுகிறது..."};
const highlighted_text = {"highlighted_text":"சிறப்பிக்கப்பட்ட உரை"};
const image = {"allow_webcam_access":"தயவுசெய்து பதிவு செய்ய வெப்கேம் அணுகலை அனுமதிக்கவும்.","brush_color":"தூரிகை நிறம்","brush_radius":"தூரிகை அளவு","image":"படம்","remove_image":"படத்தை நீக்கு","select_brush_color":"தூரிகை நிறத்தைத் தேர்ந்தெடுக்கவும்","start_drawing":"வரைய தொடங்கவும்","use_brush":"தூரிகையைப் பயன்படுத்தவும்","drop_to_upload":"பட கோப்பை பதிவேற்ற இங்கே விடவும்"};
const label = {"label":"லேபிள்"};
const login = {"enable_cookies":"நீங்கள் மறைநிலை பயன்முறையில் HuggingFace Space ஐப் பார்வையிட்டால், மூன்றாம் தரப்பு குக்கீகளை இயக்க வேண்டும்.","incorrect_credentials":"தவறான சான்றுகள்","username":"பயனர்பெயர்","password":"கடவுச்சொல்","login":"உள்நுழைய"};
const number = {"number":"எண்"};
const plot = {"plot":"வரைபடம்"};
const radio = {"radio":"ரேடியோ பொத்தான்"};
const slider = {"slider":"சறுக்கி"};
const video = {"drop_to_upload":"வீடியோ கோப்பை பதிவேற்ற இங்கே விடவும்"};
const chatbot = {"edit":"திருத்து","retry":"மீண்டும் முயற்சி","undo":"செயல்தவிர்","submit":"சமர்ப்பிக்கவும்","cancel":"ரத்து","like":"விரும்புகிறேன்","dislike":"விரும்பவில்லை","clear":"உரையாடலை அழி"};
const ta = {
  _name,
  common,
  upload_text,
  "3D_model": {"3d_model":"3D மாதிரி","drop_to_upload":"3D மாதிரி கோப்பை (.obj, .glb, .stl, .gltf, .splat, அல்லது .ply) பதிவேற்ற இங்கே விடவும்"},
  annotated_image,
  audio,
  blocks,
  checkbox,
  code,
  color_picker,
  dataframe,
  dropdown,
  errors,
  file,
  highlighted_text,
  image,
  label,
  login,
  number,
  plot,
  radio,
  slider,
  video,
  chatbot,
};

export { _name, annotated_image, audio, blocks, chatbot, checkbox, code, color_picker, common, dataframe, ta as default, dropdown, errors, file, highlighted_text, image, label, login, number, plot, radio, slider, upload_text, video };
//# sourceMappingURL=ta-BTn6NxAt.js.map
