import { _ as __name } from './mermaid.core-m0ZQP3yq.js';

// src/diagrams/globalStyles.ts
var getIconStyles = /* @__PURE__ */ __name(() => `
  /* Font Awesome icon styling - consolidated */
  .label-icon {
    display: inline-block;
    height: 1em;
    overflow: visible;
    vertical-align: -0.125em;
  }
  
  .node .label-icon path {
    fill: currentColor;
    stroke: revert;
    stroke-width: revert;
  }
`, "getIconStyles");

export { getIconStyles as g };
//# sourceMappingURL=chunk-FMBD7UC4-hWqaK4zc.js.map
