
	svg.svelte-1vhirvf {
		width: var(--size-20);
		height: var(--size-20);
	}

	svg.svelte-1vhirvf path:where(.svelte-1vhirvf) {
		fill: var(--loader-color);
	}

	div.svelte-1vhirvf {
		z-index: var(--layer-2);
	}

	.margin.svelte-1vhirvf {
		margin: var(--size-4);
	}
	.wrap.svelte-1uj8rng {
		display: flex;
		flex-direction: column;
		justify-content: center;
		align-items: center;
		z-index: var(--layer-3);
		transition: opacity 0.1s ease-in-out;
		border-radius: var(--block-radius);
		background: var(--block-background-fill);
		padding: 0 var(--size-6);
		overflow: hidden;
	}
	.no-click.svelte-1uj8rng {
		pointer-events: none;
	}

	.wrap.center.svelte-1uj8rng {
		top: 0;
		right: 0px;
		left: 0px;
	}

	.wrap.default.svelte-1uj8rng {
		top: 0px;
		right: 0px;
		bottom: 0px;
		left: 0px;
	}

	.hide.svelte-1uj8rng {
		opacity: 0;
		pointer-events: none;
	}

	.generating.svelte-1uj8rng {
		animation:
			svelte-1uj8rng-pulseStart 1s cubic-bezier(0.4, 0, 0.6, 1),
			svelte-1uj8rng-pulse 2s cubic-bezier(0.4, 0, 0.6, 1) 1s infinite;
		border: 2px solid var(--color-accent);
		background: transparent;
		z-index: var(--layer-1);
		pointer-events: none;
	}

	.translucent.svelte-1uj8rng {
		background: none;
	}

	@keyframes svelte-1uj8rng-pulseStart {
		0% {
			opacity: 0;
		}
		100% {
			opacity: 1;
		}
	}

	@keyframes svelte-1uj8rng-pulse {
		0%,
		100% {
			opacity: 1;
		}
		50% {
			opacity: 0.5;
		}
	}

	.loading.svelte-1uj8rng {
		z-index: var(--layer-2);
		color: var(--body-text-color);
	}
	.eta-bar.svelte-1uj8rng {
		position: absolute;
		top: 0;
		right: 0;
		bottom: 0;
		left: 0;
		transform-origin: left;
		opacity: 0.8;
		z-index: var(--layer-1);
		transition: 10ms;
		background: var(--background-fill-secondary);
	}
	.progress-bar-wrap.svelte-1uj8rng {
		border: 1px solid var(--border-color-primary);
		background: var(--background-fill-primary);
		width: 55.5%;
		height: var(--size-4);
	}
	.progress-bar.svelte-1uj8rng {
		transform-origin: left;
		background-color: var(--loader-color);
		width: var(--size-full);
		height: var(--size-full);
	}

	.progress-level.svelte-1uj8rng {
		display: flex;
		flex-direction: column;
		align-items: center;
		gap: 1;
		z-index: var(--layer-2);
		width: var(--size-full);
	}

	.progress-level-inner.svelte-1uj8rng {
		margin: var(--size-2) auto;
		color: var(--body-text-color);
		font-size: var(--text-sm);
		font-family: var(--font-mono);
	}

	.meta-text.svelte-1uj8rng {
		position: absolute;
		bottom: 0;
		right: 0;
		z-index: var(--layer-2);
		padding: var(--size-1) var(--size-2);
		font-size: var(--text-sm);
		font-family: var(--font-mono);
	}

	.meta-text-center.svelte-1uj8rng {
		display: flex;
		position: absolute;
		top: 0;
		right: 0;
		justify-content: center;
		align-items: center;
		transform: translateY(var(--size-6));
		z-index: var(--layer-2);
		padding: var(--size-1) var(--size-2);
		font-size: var(--text-sm);
		font-family: var(--font-mono);
		text-align: center;
	}

	.error.svelte-1uj8rng {
		box-shadow: var(--shadow-drop);
		border: solid 1px var(--error-border-color);
		border-radius: var(--radius-full);
		background: var(--error-background-fill);
		padding-right: var(--size-4);
		padding-left: var(--size-4);
		color: var(--error-text-color);
		font-weight: var(--weight-semibold);
		font-size: var(--text-lg);
		line-height: var(--line-lg);
		font-family: var(--font);
	}

	.validation-error.svelte-1uj8rng {
		pointer-events: auto;
		color: var(--error-text-color);
		font-weight: var(--weight-semibold);
		font-size: var(--text-lg);
		line-height: var(--line-lg);
		font-family: var(--font);
		position: absolute;
		background: var(--error-background-fill);
		top: 0;
		right: 0;
		z-index: var(--layer-3);
		padding: var(--size-1) var(--size-2);
		font-size: var(--text-md);
		text-align: center;
		border-bottom-left-radius: var(--radius-sm);
		border-bottom: 1px solid var(--error-border-color);
		border-left: 1px solid var(--error-border-color);
		display: flex;
		justify-content: space-between;
		align-items: center;
		gap: var(--spacing-xl);
	}

	.minimal.svelte-1uj8rng {
		pointer-events: none;
	}

	.minimal.svelte-1uj8rng .progress-text:where(.svelte-1uj8rng) {
		background: var(--block-background-fill);
	}

	.border.svelte-1uj8rng {
		border: 1px solid var(--border-color-primary);
	}

	.clear-status.svelte-1uj8rng {
		position: absolute;
		display: flex;
		top: var(--size-2);
		right: var(--size-2);
		justify-content: flex-end;
		gap: var(--spacing-sm);
		z-index: var(--layer-1);
	}
	.toast-body.svelte-irmu64 {
		display: flex;
		flex-direction: column;
		position: relative;
		right: 0;
		left: 0;
		margin: auto;
		border-radius: var(--radius-sm);
		overflow: hidden;
		pointer-events: auto;
		background: var(--background-fill-secondary);
		border: 1px solid var(--border-color-primary);
		box-shadow: var(--shadow-drop);
		touch-action: pan-y;
		user-select: none;
	}

	.dark .toast-body.svelte-irmu64 {
		background: var(--background-fill-primary);
	}

	.toast-body.error.svelte-irmu64 {
		--toast-color: var(--color-red-700);
	}

	.toast-body.warning.svelte-irmu64 {
		--toast-color: var(--color-yellow-700);
	}

	.toast-body.info.svelte-irmu64 {
		--toast-color: var(--color-grey-700);
	}

	.toast-body.success.svelte-irmu64 {
		--toast-color: var(--color-green-700);
	}

	.dark .toast-body.error.svelte-irmu64 {
		--toast-color: var(--color-red-500);
	}

	.dark .toast-body.warning.svelte-irmu64 {
		--toast-color: var(--color-yellow-500);
	}

	.dark .toast-body.info.svelte-irmu64 {
		--toast-color: var(--color-grey-500);
	}

	.dark .toast-body.success.svelte-irmu64 {
		--toast-color: var(--color-green-500);
	}

	.toast-header.svelte-irmu64 {
		display: flex;
		align-items: center;
		padding: var(--size-3) var(--size-3);
		cursor: pointer;
	}

	.toast-header.svelte-irmu64:hover {
		opacity: 0.9;
	}

	.toast-title-row.svelte-irmu64 {
		flex: 1;
		display: flex;
		align-items: center;
		gap: var(--size-2);
	}

	.chevron.svelte-irmu64 {
		display: flex;
		align-items: center;
		width: var(--size-4);
		height: var(--size-4);
		transition: transform 0.2s ease;
		opacity: 0.6;
	}

	.chevron.visible.svelte-irmu64 {
		opacity: 1;
	}

	.chevron.expanded.svelte-irmu64 {
		transform: rotate(180deg);
	}

	.toast-title.svelte-irmu64 {
		display: flex;
		align-items: center;
		font-weight: var(--weight-bold);
		font-size: var(--text-lg);
		line-height: var(--line-sm);
	}

	.toast-count.svelte-irmu64 {
		margin-left: var(--size-1);
		font-weight: var(--weight-semibold);
	}

	.toast-close.svelte-irmu64 {
		margin: 0 var(--size-1);
		border-radius: var(--radius-lg);
		padding: var(--size-1);
		font-size: var(--text-xl);
		line-height: 1;
		background: transparent;
		border: none;
		cursor: pointer;
		transition: all 0.2s ease;
		display: flex;
		align-items: center;
		justify-content: center;
		width: var(--size-7);
		height: var(--size-7);
		opacity: 0.6;
		flex-shrink: 0;
	}

	.toast-close.svelte-irmu64:hover {
		opacity: 1;
		transform: scale(1.1);
	}

	.toast-close.svelte-irmu64:active {
		transform: scale(0.95);
	}

	.toast-close.svelte-irmu64 {
		color: var(--body-text-color);
	}

	.toast-icon.svelte-irmu64 {
		display: flex;
		position: relative;
		flex-shrink: 0;
		justify-content: center;
		align-items: center;
		margin-right: var(--size-2);
		border-radius: var(--radius-full);
		padding: var(--size-1-5);
		width: var(--size-8);
		height: var(--size-8);
	}

	.toast-icon.svelte-irmu64 {
		color: var(--toast-color);
	}

	.toast-messages.svelte-irmu64 {
		display: flex;
		flex-direction: column;
		gap: var(--size-2);
		padding: 0 var(--size-3) var(--size-2) var(--size-3);
	}

	.toast-message-item.svelte-irmu64 {
		padding: var(--size-1-5) var(--size-1-5) var(--size-1-5) var(--size-1-5);
		border-radius: var(--radius-lg);
	}

	.toast-separator.svelte-irmu64 {
		height: 1px;
		background: var(--border-color-primary);
		margin: 0;
	}

	.toast-message-text.svelte-irmu64 {
		font-size: var(--text-md);
		line-height: 1.5;
		word-wrap: break-word;
		overflow-wrap: break-word;
		word-break: break-word;
		color: var(--body-text-color);
	}

	.toast-message-text.svelte-irmu64 a {
		text-decoration: underline;
	}

	@keyframes svelte-irmu64-countdown {
		from {
			transform: scaleX(1);
		}
		to {
			transform: scaleX(0);
		}
	}

	.timer.svelte-irmu64 {
		position: absolute;
		bottom: 0;
		left: 0;
		transform-origin: 0 0;
		animation: svelte-irmu64-countdown 10s linear forwards;
		width: 100%;
		height: var(--size-1);
	}

	.timer.svelte-irmu64 {
		background: var(--toast-color);
	}

	@media (max-width: 640px) {
		.toast-header.svelte-irmu64 {
			padding: var(--size-2) var(--size-2);
		}

		.toast-messages.svelte-irmu64 {
			gap: var(--size-1);
			padding: 0 var(--size-2) var(--size-1-5) var(--size-2);
		}

		.toast-message-item.svelte-irmu64 {
			padding: 0 var(--size-1) var(--size-1) var(--size-1);
		}
	}
	.toast-wrap.svelte-1qhecvt {
		--toast-top: var(--size-4);
		display: flex;
		position: fixed;
		top: calc(var(--toast-top) + var(--size-3));
		flex-direction: column;
		gap: var(--size-2);
		z-index: var(--layer-top);
		right: var(--size-3);
		left: var(--size-3);
		align-items: end;
		max-width: none;
	}

	.toast-item.svelte-1qhecvt {
		width: 100%;
	}

	@media (min-width: 640px) {
		.toast-wrap.svelte-1qhecvt {
			left: auto;
			width: calc(var(--size-96) + var(--size-10));
		}
	}
	.streaming-bar.svelte-1au5sp1 {
		position: absolute;
		bottom: 0;
		left: 0;
		right: 0;
		height: 4px;
		background-color: var(--primary-600);
		animation: svelte-1au5sp1-countdown linear forwards;
		z-index: 1;
	}

	@keyframes svelte-1au5sp1-countdown {
		from {
			transform: translateX(0%);
		}
		to {
			transform: translateX(-100%);
		}
	}