
	button.svelte-1lrnmgy {
		display: flex;
		justify-content: center;
		align-items: center;
		gap: 1px;
		z-index: var(--layer-2);
		/* background: var(--background-fill-primary); */
		/* border-radius: var(--radius-sm); */
		color: var(--block-label-text-color);
		border: 1px solid transparent;
	}

	.quite-round.svelte-1lrnmgy {
		border-radius: var(--radius-sm);
	}

	.very-round.svelte-1lrnmgy {
		border-radius: var(--radius-md);
	}

	button[disabled].svelte-1lrnmgy {
		opacity: 0.5;
		box-shadow: none;
	}

	button[disabled].svelte-1lrnmgy:hover {
		cursor: not-allowed;
		/* border: 1px solid var(--button-secondary-border-color); */
		/* padding: 2px; */
	}

	.padded.svelte-1lrnmgy {
		padding: 2px;
		background: var(--bg-color);
		box-shadow: var(--shadow-drop);
		border: 1px solid var(--button-secondary-border-color);
	}

	button.svelte-1lrnmgy:hover,
	button.highlight.svelte-1lrnmgy {
		cursor: pointer;
		color: var(--color-accent);
	}

	.padded.svelte-1lrnmgy:hover {
		border: 2px solid var(--button-secondary-border-color-hover);
		padding: 1px;
		color: var(--block-label-text-color);
	}

	span.svelte-1lrnmgy {
		padding: 0px 1px;
		font-size: 10px;
	}

	div.svelte-1lrnmgy {
		padding: 2px;
		display: flex;
		align-items: flex-end;
	}

	.small.svelte-1lrnmgy {
		width: 14px;
		height: 14px;
	}

	.medium.svelte-1lrnmgy {
		width: 20px;
		height: 20px;
	}

	.large.svelte-1lrnmgy {
		width: 22px;
		height: 22px;
	}

	.pending.svelte-1lrnmgy {
		animation: svelte-1lrnmgy-flash 0.5s infinite;
	}

	@keyframes svelte-1lrnmgy-flash {
		0% {
			opacity: 0.5;
		}
		50% {
			opacity: 1;
		}
		100% {
			opacity: 0.5;
		}
	}

	.transparent.svelte-1lrnmgy {
		background: transparent;
		border: none;
		box-shadow: none;
	}
	.hue-slider.svelte-qx84u5 {
		position: relative;
		width: 90%;
		margin: 0px auto 10px auto;
		height: 12px;
		border-radius: 2px;
		border: 1px solid var(--block-border-color);
		background: linear-gradient(
			to right,
			hsl(0, 100%, 50%) 0%,
			#ff0 17%,
			lime 33%,
			cyan 50%,
			blue 67%,
			magenta 83%,
			red 100%
		);
		border-top: none;
	}

	.opacity-slider.svelte-qx84u5 {
		position: relative;
		width: 90%;
		margin: 10px auto 5px auto;
		height: 12px;
		border-radius: 2px;
		border: 1px solid var(--block-border-color);
		/* background: linear-gradient(45deg, #808080 25%, transparent 25%),
			linear-gradient(-45deg, #808080 25%, transparent 25%),
			linear-gradient(45deg, transparent 75%, #808080 75%),
			linear-gradient(-45deg, transparent 75%, #808080 75%); */
		background:
			linear-gradient(to right, transparent, var(--color)),
			repeating-conic-gradient(#808080 0% 25%, transparent 0% 50%) 0 0 / 10px
				10px;
	}

	.opacity-marker.svelte-qx84u5 {
		position: absolute;
		width: 16px;
		height: 16px;
		border-radius: 50%;
	}

	.opacity-slider.svelte-qx84u5 > .opacity-marker:where(.svelte-qx84u5) {
		box-shadow: 0 0 1px #888;
		top: -4px;
		margin: auto;
		height: 18px;
		width: 18px;
		left: -9px;
		margin: auto;
		border: 1px solid var(--block-border-color);
	}

	.color-gradient.svelte-qx84u5 {
		position: relative;
		--hue: white;

		background:
			linear-gradient(rgba(0, 0, 0, 0), #000),
			linear-gradient(90deg, #fff, hsl(var(--hue), 100%, 50%));
		width: 100%;
		height: 150px;
		border-top-left-radius: 4px;
		border-top-right-radius: 4px;
	}

	.marker.svelte-qx84u5 {
		position: absolute;
		width: 14px;
		height: 14px;
		border-radius: 50%;
		border: 2px solid white;

		top: -7px;
		left: -7px;
		box-shadow: 0 1px 5px rgba(0, 0, 0, 0.1);
		pointer-events: none;
		border: 1px solid var(--block-border-color);
	}

	.hue-slider.svelte-qx84u5 > .marker:where(.svelte-qx84u5) {
		box-shadow: 0 0 1px #888;
		top: -3px;
		margin: auto;
		height: 18px;
		width: 18px;
		left: -9px;

		border: 1px solid var(--block-border-color);
	}
	.swatch-wrap.svelte-q8yof7 {
		display: flex;
		justify-content: space-between;
		align-items: flex-start;
		/* max-width: 200px; */
		width: 205px;
	}

	span.lg.svelte-q8yof7 {
		margin-top: var(--spacing-sm);
	}
	.swatch-container.svelte-q8yof7 {
		display: flex;
		flex-direction: column;
		gap: 10px;
		width: 100%;
	}
	.swatch.svelte-q8yof7 {
		display: grid;
		grid-template-columns: repeat(auto-fill, 25px);
		gap: 8px;
		justify-content: flex-start;

		overflow: hidden;
		justify-content: center;
	}

	.empty.svelte-q8yof7 {
		border: 1px solid var(--block-border-color);
		font-weight: var(--weight-bold);
		display: flex;
		justify-content: center;
		align-items: center;
		padding-top: 4px;
		text-align: center;
		font-size: var(--scale-0);
		cursor: pointer;
	}

	.empty.svelte-q8yof7::after {
		content: "+";
		margin-bottom: var(--size-1);
	}

	.color.svelte-q8yof7 {
		width: 25px;
		height: 25px;
		border-radius: var(--radius-md);
	}

	.empty.selected.svelte-q8yof7,
	.color.selected.svelte-q8yof7 {
		border: 2px solid var(--color-accent);
	}

	.colorpicker.svelte-q8yof7 {
		border: 1px solid var(--block-border-color);

		background: conic-gradient(
			red,
			#ff0 60deg,
			lime 120deg,
			cyan 180deg,
			blue 240deg,
			magenta 300deg,
			red 360deg
		);
		overflow: hidden;
		position: relative;
		color: white;
		cursor: pointer;
	}

	.colorpicker.svelte-q8yof7::before {
		content: "";
		position: absolute;
		width: 100%;
		height: 100%;
		background: rgba(0, 0, 0, 0.6);
		opacity: 1;
		transition: 0.1s;
		top: 0;
		left: 0;
	}

	.colorpicker.svelte-q8yof7::after {
		display: flex;
		justify-content: center;
		align-items: center;
		content: "⨯";
		color: white;
		font-size: var(--scale-5);
		position: absolute;
		width: 100%;
		height: 100%;
		transform: translateY(-7px);
		opacity: 1;
		transition: 0.1s;
		top: 0;
	}

	.colorpicker.hidden.svelte-q8yof7::after,
	.colorpicker.hidden.svelte-q8yof7::before {
		opacity: 0;
	}
	.input.svelte-yn52fn {
		display: flex;
		align-items: center;
		padding: 0 10px 15px;
	}

	.input.svelte-yn52fn input:where(.svelte-yn52fn) {
		height: 30px;
		width: 100%;
		flex-shrink: 1;
		border-bottom-left-radius: 0;
		border: 1px solid var(--block-border-color);
		letter-spacing: -0.05rem;
		border-left: none;
		border-right: none;
		font-family: var(--font-mono);
		font-size: var(--scale-000);
		padding-left: 15px;
		padding-right: 0;
		background-color: var(--background-fill-secondary);
		color: var(--block-label-text-color);
	}

	.swatch.svelte-yn52fn {
		width: 50px;
		height: 50px;
		border-top-left-radius: 15px;
		border-bottom-left-radius: 15px;
		flex-shrink: 0;
		border: 1px solid var(--block-border-color);
	}

	.buttons.svelte-yn52fn button:where(.svelte-yn52fn) {
		display: flex;
		align-items: center;
		justify-content: center;
		border: 1px solid var(--block-border-color);
		background: var(--background-fill-secondary);
		padding: 3px 6px;
		font-size: var(--scale-000);
		cursor: pointer;
		border-right: none;
		width: 100%;
		border-top: none;
	}

	.buttons.svelte-yn52fn button:where(.svelte-yn52fn):first-child {
		border-left: none;
	}

	.buttons.svelte-yn52fn button:where(.svelte-yn52fn):last-child {
		border-bottom-right-radius: 15px;
		border-right: 1px solid var(--block-border-color);
	}

	.buttons.svelte-yn52fn button:where(.svelte-yn52fn):hover {
		background: var(--background-fill-secondary-hover);
		font-weight: var(--weight-bold);
	}

	.buttons.svelte-yn52fn button.active:where(.svelte-yn52fn) {
		background: var(--background-fill-secondary);
		font-weight: var(--weight-bold);
	}

	.buttons.svelte-yn52fn {
		height: 20px;

		display: flex;
		justify-content: stretch;
		gap: 0px;
		/* padding: 0 10px; */
	}

	.input-wrap.svelte-yn52fn {
		display: flex;
	}

	.eyedropper.svelte-yn52fn {
		display: flex;
		align-items: center;
		justify-content: center;
		width: 25px;
		height: 30px;
		border-top-right-radius: 15px;
		border: 1px solid var(--block-border-color);
		border-left: none;
		background: var(--background-fill-secondary);
		height: 30px;
		padding: 7px 7px 5px 0px;
		cursor: pointer;
	}
	.wrap.svelte-1y25b79 {
		width: 230px;
		display: flex;
		gap: var(--size-4);
		background: var(--background-fill-secondary);
		padding: 0 var(--size-4);
		cursor: default;
		padding-top: var(--size-2-5);
	}

	input.svelte-1y25b79 {
		width: 100%;
	}
	span.svelte-1y25b79 {
		width: 26px;
		color: var(--body-text-color);
	}
	.wrap.svelte-1l3f0ck {
		max-width: 230px;
		/* width: 90%; */
		position: absolute;
		display: flex;
		flex-direction: column;
		gap: 5px;
		background: var(--background-fill-secondary);
		border: 1px solid var(--block-border-color);
		border-radius: var(--radius-md);
		box-shadow: 0px 0px 5px 0px rgba(0, 0, 0, 0.1);
		padding-bottom: var(--size-2);
		pointer-events: all;
		cursor: default;
		z-index: var(--layer-top);
		overflow: hidden;
		top: 0;

		transform: translate(35px, -12px);
	}

	.color_picker.svelte-1l3f0ck {
		transform: translate(35px, -50%);
	}

	.size_picker.svelte-1l3f0ck {
		transform: translate(35px, 18px);
	}

	.eraser_picker.svelte-1l3f0ck {
		transform: translate(35px, -7px);
	}
	.toolbar-wrap.svelte-1uf24la {
		display: inline-flex;
		flex-direction: column;
		justify-content: center;
		align-items: center;
		position: absolute;
		left: 15px;
		right: 0;
		top: 30%;
		/* transform: translateY(-50%); */
		width: 30px;
		border: 1px solid var(--block-border-color);
		border-radius: var(--radius-md);

		gap: var(--spacing-sm);
		z-index: 1000;
		background-color: var(--block-background-fill);
	}

	.half-container.svelte-1uf24la {
		flex: 1;
		display: flex;
		flex-direction: column;
		gap: var(--spacing-sm);
		position: relative;
	}

	.half-container.svelte-1uf24la {
		width: 100%;
		padding: var(--spacing-sm) 0;
	}

	.half-container.svelte-1uf24la:last-child {
		border-top: 1px solid var(--block-border-color);
	}

	.hide.svelte-1uf24la {
		display: none;
	}

	/* .toolbar-left,
	.toolbar-right {
		display: flex;
		flex-direction: row;
		align-items: center;
		gap: var(--spacing-sm);
		display: flex;
		z-index: 1000;
		border-radius: var(--radius-sm);
		background-color: #fff;
		padding: var(--spacing-sm) 0.3rem;
		width: auto;
	}

	.toolbar-right {
		border-left: 0;
		border-top-left-radius: 0;
		border-bottom-left-radius: 0;
	}

	.toolbar-left {
		border-top-right-radius: 0;
		border-bottom-right-radius: 0;
	}

	.toolbar-layers {
		border-top-left-radius: 0;
		border-bottom-left-radius: 0;
	} */
	.add-layer.svelte-189k4ah {
		display: flex;
		justify-content: center;
		align-items: center;
		margin: var(--spacing-sm) 0 0 0;
	}

	.add-layer.svelte-189k4ah button {
		width: 100%;
	}
	.icon.svelte-189k4ah {
		width: 14px;
		color: var(--body-text-color);
		transition: color 0.15s ease;
	}

	.layer-wrap.svelte-189k4ah {
		/* position: absolute; */
		display: flex;
		justify-content: center;
		align-items: center;
		/* border: 1px solid var(--border-color-primary); */
		/* z-index: var(--layer-1); */
		/* border-radius: var(--radius-sm); */
		/* background-color: var(--block-background-fill); */
		width: 100%;
		flex-direction: column;
		/* bottom: 5px; */
		/* right: 5px; */
		/* border-bottom: 0;
		border-right: 0; */
		/* box-shadow: var(--shadow-drop); */
	}

	.layer-wrap.svelte-189k4ah button:where(.svelte-189k4ah) {
		justify-content: space-between;
		align-items: center;
		width: 100%;
		display: flex;
		font-size: var(--text-sm);
		line-height: var(--line-sm);
		transition: all 0.15s ease;
		padding: var(--spacing-md) var(--spacing-lg);
		/* padding-right: var(--spacing-lg); */
		border-radius: var(--radius-sm);
		/* padding-left: 30px; */
	}

	button.layer-title-button.svelte-189k4ah {
		width: 100%;
		/* padding-left: 30px; */
	}

	.layer-wrap.svelte-189k4ah button:where(.svelte-189k4ah):hover .icon:where(.svelte-189k4ah) {
		color: var(--color-accent);
	}

	.selected_layer.svelte-189k4ah {
		color: var(--color-accent);
		font-weight: var(--weight-semibold);
	}

	ul.svelte-189k4ah {
		/* position: absolute; */
		bottom: 0;
		right: 0;
		/* background: var(--block-background-fill); */
		width: 100%;
		/* min-width: 100%; */
		list-style: none;
		z-index: var(--layer-top);
		/* border: 1px solid var(--border-color-primary); */
		/* border-radius: var(--radius-sm); */
		/* box-shadow: var(--shadow-drop-lg); */
		padding: var(--spacing-sm);
		/* transform: translate(-1px, 1px); */
		border-top: 1px solid var(--border-color-primary);
	}

	/* .sep {
		height: 12px;
		background-color: var(--block-border-color);
		width: 1px;
		display: block;
		margin-left: var(--spacing-xl);
	} */

	li.svelte-189k4ah {
		display: flex;
		justify-content: space-between;
		align-items: center;
		/* border-radius: var(--radius-sm); */
		transition: background-color 0.15s ease;
		border-bottom: 1px solid var(--border-color-primary);
	}

	.layer-wrap.svelte-189k4ah ul:where(.svelte-189k4ah) > li:where(.svelte-189k4ah) > button:where(.svelte-189k4ah) {
		padding-left: calc(var(--spacing-md));
	}

	li.svelte-189k4ah:hover {
		background-color: var(--background-fill-secondary);
	}

	li.svelte-189k4ah:last-child {
		border-bottom: none;
	}
	li.svelte-189k4ah:last-child:hover {
		background-color: unset;
	}

	li.svelte-189k4ah div:where(.svelte-189k4ah) {
		display: flex;
		gap: var(--spacing-sm);
		padding: var(--spacing-sm);
	}
	li.svelte-189k4ah > button:where(.svelte-189k4ah) {
		padding: var(--spacing-sm) var(--spacing-md);
	}

	/* li > div > button {
		opacity: 0.7;
		transition: opacity 0.15s ease;
	}

	li > div > button:hover {
		opacity: 1;
	} */
	.toolbar-wrap-wrap.svelte-1357rcu {
		position: absolute;
		right: 0;
		bottom: 0;
		width: fit-content;
		max-height: 100%;
		overflow-y: auto;
		display: flex;
		flex-direction: column-reverse;
	}

	.toolbar-wrap.svelte-1357rcu {
		min-width: 110px;
		display: flex;
		flex-direction: column;
		justify-content: center;
		align-items: center;
		position: relative;
		right: 0;
		bottom: 0;
		border: 1px solid var(--block-border-color);
		border-radius: 0;
		border-top-left-radius: var(--radius-md);
		border-right: none;
		border-bottom: none;
		z-index: 1000;
		background-color: var(--block-background-fill);
	}
	.separator.svelte-o0h5ih {
		width: 1px;
		height: 10px;
		background-color: var(--border-color-primary);
	}

	.zoom-number.svelte-o0h5ih {
		position: relative;
		width: 30px;
		padding-left: 4px;
		display: flex;
		align-items: center;
		justify-content: center;
	}

	.zoom-number.svelte-o0h5ih span:where(.svelte-o0h5ih) {
		cursor: pointer;
		font-size: var(--text-sm);
		transition: color 0.15s ease;
	}

	.zoom-number.svelte-o0h5ih span:where(.svelte-o0h5ih):hover {
		color: var(--color-accent);
	}

	.zoom-controls.svelte-o0h5ih {
		position: absolute;
		top: calc(100% + var(--spacing-xxs) + 3px);
		left: -2px;
		background: var(--block-background-fill);
		border: 1px solid var(--color-gray-200);
		border-radius: var(--radius-sm);
		padding: var(--spacing-xxs);
		box-shadow: var(--shadow-drop);
		font-size: 12px;
		z-index: var(--layer-2);
		width: max-content;
		color: var(--block-label-text-color);
		border: 1px solid var(--block-border-color);
		border-top-left-radius: 0;
		border-top-right-radius: 0;
	}

	.zoom-controls.svelte-o0h5ih ul:where(.svelte-o0h5ih) {
		list-style: none;
		margin: 0;
		padding: 0;
	}

	.zoom-controls.svelte-o0h5ih li:where(.svelte-o0h5ih) {
		display: flex;
		justify-content: space-between;
		align-items: center;
		transition: background-color 0.15s ease;
		border-bottom: 1px solid var(--border-color-primary);
	}

	.zoom-controls.svelte-o0h5ih li:where(.svelte-o0h5ih):last-child {
		border-bottom: none;
	}

	.zoom-controls.svelte-o0h5ih li:where(.svelte-o0h5ih):hover {
		background-color: var(--background-fill-secondary);
	}

	.zoom-controls.svelte-o0h5ih button:where(.svelte-o0h5ih) {
		width: 100%;
		text-align: left;
		padding: var(--spacing-sm) var(--spacing-md);
		font-size: var(--text-sm);
		line-height: var(--line-sm);
		transition: all 0.15s ease;
		border-radius: var(--radius-sm);
	}

	.zoom-controls.svelte-o0h5ih button:where(.svelte-o0h5ih):hover {
		color: var(--color-accent);
	}
	.image-container.svelte-6fnl1z {
		display: flex;
		height: 100%;
		flex-direction: column;
		justify-content: center;
		align-items: center;
		max-height: 100%;
		border-radius: var(--radius-sm);
	}

	.pixi-target.svelte-6fnl1z {
		width: 100%;
		height: 100%;
		position: absolute;
		top: 0;
		left: 0;
		z-index: 1;
		display: block;
		opacity: 0;
		pointer-events: none;
		border-radius: var(--radius-sm);
	}

	.pixi-target-crop.svelte-6fnl1z {
		width: 100%;
		height: 100%;
		position: absolute;
		top: 0;
		left: 0;
		z-index: 2;
		display: block;
		opacity: 0;
		pointer-events: none;
		border-radius: var(--radius-sm);
	}

	.visible.svelte-6fnl1z {
		opacity: 1;
		pointer-events: auto;
	}

	.pixi-target.svelte-6fnl1z {
		width: 100%;
		height: 100%;
		position: absolute;
		top: 0;
		left: 0;
		right: 0;
		bottom: 0;
		overflow: hidden;
	}

	.modal.svelte-6fnl1z {
		position: absolute;
		height: 100%;
		width: 100%;
		left: 0;
		right: 0;
		margin: auto;
		z-index: var(--layer-top);
		display: flex;
		align-items: center;
	}

	.modal-inner.svelte-6fnl1z {
		height: 100%;
		width: 100%;
		background: var(--block-background-fill);
	}

	.dark-bg.svelte-6fnl1z {
		background: #333;
	}

	.crop-confirm-button.svelte-6fnl1z {
		position: absolute;
		bottom: 8px;
		left: 0;
		right: 0;
		margin: auto;
		z-index: var(--layer-top);
		display: flex;
		align-items: center;
		justify-content: center;
	}
	h2.svelte-b4yllx {
		font-size: var(--text-xl);
	}

	p.svelte-b4yllx,
	h2.svelte-b4yllx {
		white-space: pre-line;
	}

	.empty.svelte-b4yllx {
		display: flex;
		flex-direction: column;
		justify-content: center;
		align-items: center;
		position: absolute;
		height: 100%;
		width: 100%;
		left: 0;
		right: 0;
		margin: auto;
		z-index: var(--layer-1);
		text-align: center;
		color: var(--color-grey-500) !important;
		cursor: pointer;
	}

	.wrap.svelte-b4yllx {
		display: flex;
		flex-direction: column;
		justify-content: center;
		align-items: center;
		line-height: var(--line-md);
		font-size: var(--text-md);
	}

	.or.svelte-b4yllx {
		color: var(--body-text-color-subdued);
	}