import { S as ShaderStore } from './index-CxR0Yf_7.js';

// Do not edit.
const name = "mainUVVaryingDeclaration";
const shader = `#ifdef MAINUV{X}
varying vec2 vMainUV{X};
#endif
`;
// Sideeffect
if (!ShaderStore.IncludesShadersStore[name]) {
    ShaderStore.IncludesShadersStore[name] = shader;
}
//# sourceMappingURL=mainUVVaryingDeclaration-Y0U6swm8.js.map
