
	div.svelte-dmtrd3 {
		display: flex;
		flex-direction: column;
		position: relative;
		border: none;
		border-radius: var(--radius-sm);
		width: 100%;
		box-sizing: border-box;
	}
	.grow-children.svelte-dmtrd3 > .column > .column {
		flex-grow: 1;
	}