import { U as Utils, C as Color } from './mermaid.core-m0ZQP3yq.js';

/* IMPORT */
/* MAIN */
const channel = (color, channel) => {
    return Utils.lang.round(Color.parse(color)[channel]);
};

export { channel as c };
//# sourceMappingURL=channel-DsT53NIR.js.map
