
	.altair.svelte-1i8dfv0 canvas {
		padding: 6px;
	}
	.altair.svelte-1i8dfv0 .vega-embed {
		padding: 0px !important;
	}
	.altair.svelte-1i8dfv0 .vega-actions {
		right: 0px !important;
	}
	.layout.svelte-1i8dfv0 {
		display: flex;
		flex-direction: column;
		justify-content: center;
		align-items: center;
		width: var(--size-full);
		height: var(--size-full);
		color: var(--body-text-color);
	}
	.altair.svelte-1i8dfv0 {
		display: flex;
		flex-direction: column;
		justify-content: center;
		align-items: center;
		width: var(--size-full);
		height: var(--size-full);
	}
	.caption.svelte-1i8dfv0 {
		font-size: var(--text-sm);
		margin-bottom: 6px;
	}
	#vg-tooltip-element {
		font-family: var(--font) !important;
		font-size: var(--text-xs) !important;
		box-shadow: none !important;
		background-color: var(--block-background-fill) !important;
		border: 1px solid var(--border-color-primary) !important;
		color: var(--body-text-color) !important;
	}
	#vg-tooltip-element .key {
		color: var(--body-text-color-subdued) !important;
	}