import { S as ShaderStore } from './index-CxR0Yf_7.js';

// Do not edit.
const name = "kernelBlurVaryingDeclaration";
const shader = `varying vec2 sampleCoord{X};`;
// Sideeffect
if (!ShaderStore.IncludesShadersStore[name]) {
    ShaderStore.IncludesShadersStore[name] = shader;
}
//# sourceMappingURL=kernelBlurVaryingDeclaration-CI9b7U8x.js.map
