import './async-D55cHugf.js';
import { s as slot } from './index-u8mz_F03.js';
import './escaping-CBnpiEl5.js';
import './context-CBkBucIx.js';

function _layout($$renderer, $$props) {
  $$renderer.push(`<!--[-->`);
  slot($$renderer, $$props, "default", {});
  $$renderer.push(`<!--]-->`);
}

export { _layout as default };
//# sourceMappingURL=_layout.svelte-D65H5kt2.js.map
