
	div.svelte-7xavid {
		display: flex;
		flex-wrap: wrap;
		gap: var(--layout-gap);
		width: var(--size-full);
		position: relative;
	}

	.hide.svelte-7xavid {
		display: none;
	}
	.compact.svelte-7xavid > *,
	.compact.svelte-7xavid .box {
		border-radius: 0;
	}
	.compact.svelte-7xavid,
	.panel.svelte-7xavid {
		border-radius: var(--container-radius);
		background: var(--background-fill-secondary);
		padding: var(--size-2);
	}
	.unequal-height.svelte-7xavid {
		align-items: flex-start;
	}

	.stretch.svelte-7xavid {
		align-items: stretch;
	}

	.stretch.svelte-7xavid > .column > *,
	.stretch.svelte-7xavid > .column > .form > * {
		flex-grow: 1;
		flex-shrink: 0;
	}

	div.svelte-7xavid > *,
	div.svelte-7xavid > .form > * {
		flex: 1 1 0%;
		flex-wrap: wrap;
		min-width: min(160px, 100%);
	}

	.grow-children.svelte-7xavid > .column {
		align-self: stretch;
	}