const _name = "Svenska";
const annotated_image = { "annotated_image": "Annoterad bild" };
const audio = { "allow_recording_access": "Tillåt mikrofonåtkomst för inspelning.", "audio": "Ljud", "record_from_microphone": "Spela in från mikrofon", "stop_recording": "Stoppa inspelning", "no_device_support": "Kan inte få åtkomst till medieenheter. Se till att du använder en säker källa (https) eller localhost (eller har gett ett giltigt SSL-certifikat till ssl_verify) och att du har gett webbläsaren åtkomst till din enhet.", "stop": "Stopp", "resume": "Fortsätt", "record": "Spela in", "no_microphone": "Ingen mikrofon hittades", "pause": "Paus", "play": "Spela upp", "waiting": "Väntar", "drop_to_upload": "Släpp en ljudfil här för att ladda upp" };
const blocks = { "connection_can_break": "På mobila enheter kan anslutningen brytas om fliken lämnas eller om enheten går i viloläge, och du förlorar din plats i kön.", "long_requests_queue": "Det är en lång kö med väntande förfrågningar. Duplicera detta Space för att hoppa över kön.", "lost_connection": "Anslutningen bröts eftersom sidan lämnades. Återgår till kön...", "waiting_for_inputs": "Väntar på att fil(er) ska vara klara att ladda upp, vänligen försök igen." };
const checkbox = { "checkbox": "Kryssruta", "checkbox_group": "Kryssrutgrupp" };
const code = { "code": "Kod" };
const color_picker = { "color_picker": "Färgväljare" };
const common = { "built_with": "Byggt med", "built_with_gradio": "Byggt med Gradio", "clear": "Rensa", "download": "Ladda ner", "edit": "Redigera", "empty": "Tom", "error": "Fel", "hosted_on": "Värd på", "loading": "Laddar", "logo": "Logotyp", "or": "eller", "remove": "Ta bort", "settings": "Inställningar", "share": "Dela", "submit": "Skicka", "undo": "Ångra", "no_devices": "Inga enheter hittades", "language": "Språk", "display_theme": "Visningstema", "pwa": "Progressiv webbapplikation" };
const dataframe = { "incorrect_format": "Fel format, endast CSV- och TSV-filer stöds", "new_column": "Lägg till kolumn", "new_row": "Ny rad", "add_row_above": "Lägg till rad ovanför", "add_row_below": "Lägg till rad nedanför", "add_column_left": "Lägg till kolumn till vänster", "add_column_right": "Lägg till kolumn till höger", "delete_row": "Ta bort rad", "delete_column": "Ta bort kolumn", "sort_column": "Sortera kolumn", "sort_ascending": "Sortera stigande", "sort_descending": "Sortera fallande", "drop_to_upload": "Släpp CSV- eller TSV-filer här för att importera data till dataramverk", "clear_sort": "Rensa sortering" };
const dropdown = { "dropdown": "Rullgardinsmeny" };
const errors = { "build_error": "Det finns ett byggfel", "config_error": "Det finns ett konfigurationsfel", "contact_page_author": "Kontakta sidans författare för att informera dem.", "no_app_file": "Det finns ingen app-fil", "runtime_error": "Det finns ett körtidsfel", "space_not_working": '"Space fungerar inte eftersom" {0}', "space_paused": "Space är pausat", "use_via_api": "Använd via API", "use_via_api_or_mcp": "Använd via API eller MCP" };
const file = { "uploading": "Laddar upp..." };
const highlighted_text = { "highlighted_text": "Markerad text" };
const image = { "allow_webcam_access": "Tillåt webbkameråtkomst för inspelning.", "brush_color": "Penselfärg", "brush_radius": "Penselstorlek", "image": "Bild", "remove_image": "Ta bort bild", "select_brush_color": "Välj penselfärg", "start_drawing": "Börja rita", "use_brush": "Använd pensel", "drop_to_upload": "Släpp en bildfil här för att ladda upp" };
const label = { "label": "Etikett" };
const login = { "enable_cookies": "Om du besöker ett HuggingFace Space i inkognitoläge måste du aktivera tredjepartscookies.", "incorrect_credentials": "Felaktiga inloggningsuppgifter", "username": "Användarnamn", "password": "Lösenord", "login": "Logga in" };
const number = { "number": "Nummer" };
const plot = { "plot": "Diagram" };
const radio = { "radio": "Radioknapp" };
const slider = { "slider": "Skjutreglage" };
const upload_text = { "click_to_upload": "Klicka för att ladda upp", "drop_audio": "Släpp ljud här", "drop_csv": "Släpp CSV här", "drop_file": "Släpp fil här", "drop_image": "Släpp bild här", "drop_video": "Släpp video här", "drop_gallery": "Släpp media här", "paste_clipboard": "Klistra in från urklipp" };
const video = { "drop_to_upload": "Släpp en videofil här för att ladda upp" };
const chatbot = { "edit": "Redigera", "retry": "Försök igen", "undo": "Ångra", "submit": "Skicka", "cancel": "Avbryt", "like": "Gilla", "dislike": "Ogilla", "clear": "Rensa chatten" };
const sv = {
  _name,
  "3D_model": { "3d_model": "3D-modell", "drop_to_upload": "Släpp en 3D-modellfil (.obj, .glb, .stl, .gltf, .splat eller .ply) här för att ladda upp" },
  annotated_image,
  audio,
  blocks,
  checkbox,
  code,
  color_picker,
  common,
  dataframe,
  dropdown,
  errors,
  file,
  highlighted_text,
  image,
  label,
  login,
  number,
  plot,
  radio,
  slider,
  upload_text,
  video,
  chatbot
};

export { _name, annotated_image, audio, blocks, chatbot, checkbox, code, color_picker, common, dataframe, sv as default, dropdown, errors, file, highlighted_text, image, label, login, number, plot, radio, slider, upload_text, video };
//# sourceMappingURL=sv-DzTbzExV.js.map
