
	.minimal-audio-player.svelte-15unlcf {
		display: flex;
		align-items: center;
		gap: var(--spacing-sm);
		border-radius: var(--radius-sm);
		width: var(--size-52);
		padding: var(--spacing-sm);
	}

	.play-btn.svelte-15unlcf {
		display: inline-flex;
		align-items: center;
		justify-content: center;
		padding: 0;
		border: none;
		background: none;
		color: var(--body-text-color);
		opacity: 0.7;
		cursor: pointer;
		border-radius: 50%;
		transition: all 0.2s ease;
		flex-shrink: 0;
	}

	.play-btn.svelte-15unlcf:hover {
		color: var(--color-accent);
		opacity: 1;
	}

	.play-btn.svelte-15unlcf:active {
		transform: scale(0.95);
	}

	.play-btn.svelte-15unlcf svg:where(.svelte-15unlcf) {
		width: var(--size-5);
		height: var(--size-5);
		display: block;
	}

	.waveform-wrapper.svelte-15unlcf {
		flex: 1 1 auto;
		cursor: pointer;
		width: auto;
	}

	.waveform-wrapper.svelte-15unlcf ::part(wrapper) {
		margin-bottom: 0;
	}

	.timestamp.svelte-15unlcf {
		font-size: 13px;
		font-weight: 500;
		color: var(--body-text-color);
		opacity: 0.7;
		font-variant-numeric: tabular-nums;
		flex-shrink: 0;
		min-width: 40px;
		text-align: center;
	}

	@media (prefers-reduced-motion: reduce) {
		.play-btn.svelte-15unlcf {
			transition: none;
		}
	}