
	.label-input.svelte-14ic1z3 {
		margin-top: 1px;
		margin-left: 4px;
		border: none;
		border-radius: var(--radius-xs);
		padding: 1px 5px;
		color: var(--color-white);
		font-weight: var(--weight-bold);
		font-size: var(--text-sm);
		text-transform: uppercase;
		line-height: 1;
	}

	.label-input.svelte-14ic1z3::placeholder {
		color: rgba(255, 255, 255, 0.5);
	}
	.container.svelte-1akrbo3 {
		display: flex;
		flex-direction: column;
		gap: var(--spacing-sm);
		padding: var(--block-padding);
	}

	.legend.svelte-1akrbo3,
	.score-legend.svelte-1akrbo3 {
		display: flex;
		flex-wrap: wrap;
		gap: var(--spacing-sm);
	}

	.legend-item.svelte-1akrbo3 {
		cursor: pointer;
		border: none;
		border-radius: var(--radius-xs);
		padding: 0 var(--size-2);
		font-weight: var(--weight-semibold);
		color: black;
	}

	.score-legend.svelte-1akrbo3 {
		justify-content: space-between;
		border-radius: var(--radius-xs);
		background: linear-gradient(
			to right,
			var(--color-purple),
			transparent,
			var(--color-red)
		);
		padding: var(--size-1) var(--size-2);
		font-weight: var(--weight-semibold);
	}

	.textfield.svelte-1akrbo3 {
		display: flex;
		flex-wrap: wrap;
		line-height: var(--scale-4);
		word-break: break-all;
	}

	.line-break.svelte-1akrbo3 {
		flex-basis: 100%;
		height: 0;
	}

	.token-container.svelte-1akrbo3 {
		position: relative;
	}

	.token.svelte-1akrbo3 {
		transition: 150ms;
		border-radius: var(--radius-xs);
	}

	.token.highlighted.svelte-1akrbo3 {
		cursor: pointer;
		padding: var(--size-0-5) var(--size-1);
		margin-left: var(--size-1);
		margin-right: var(--size-2);
	}

	.token.highlighted.transparent.svelte-1akrbo3 {
		padding: var(--size-0-5) var(--size-0-5);
		margin: 0;
		outline: 1px solid transparent;
	}

	.token.highlighted.transparent.svelte-1akrbo3:hover {
		outline-color: var(--neutral-400);
	}

	.dark .token.highlighted.transparent.svelte-1akrbo3:hover {
		outline-color: var(--neutral-500);
	}

	.token.dimmed.svelte-1akrbo3 {
		color: var(--body-text-color);
	}

	.text.svelte-1akrbo3 {
		white-space: pre-wrap;
	}

	.text.unlabeled.svelte-1akrbo3 {
		color: var(--body-text-color);
	}

	.score-token.svelte-1akrbo3 .text:where(.svelte-1akrbo3) {
		color: var(--body-text-color);
	}

	.label.svelte-1akrbo3 {
		margin-left: 4px;
		border-radius: var(--radius-xs);
		padding: 1px 5px;
		color: var(--color-white);
		font-weight: var(--weight-bold);
		text-transform: uppercase;
		font-size: 70%;
		vertical-align: middle;
		bottom: 1px;
		position: relative;
	}

	.remove-btn.svelte-1akrbo3 {
		display: none;
		position: absolute;
		top: 0;
		right: 0;
		width: var(--size-3);
		height: var(--size-3);
		border: none;
		border-radius: 50%;
		background: var(--neutral-400);
		color: white;
		font-size: 10px;
		cursor: pointer;
		justify-content: center;
		align-items: center;
	}

	.dark .remove-btn.svelte-1akrbo3 {
		background: var(--neutral-500);
		color: var(--neutral-950);
	}

	.remove-btn.svelte-1akrbo3 svg {
		width: var(--size-1-5);
		height: var(--size-1-5);
	}

	.remove-btn.svelte-1akrbo3:hover {
		background: var(--neutral-500);
	}

	.dark .remove-btn.svelte-1akrbo3:hover {
		background: var(--neutral-400);
	}

	.token-container.svelte-1akrbo3:hover .remove-btn:where(.svelte-1akrbo3),
	.token-container.svelte-1akrbo3:focus-within .remove-btn:where(.svelte-1akrbo3) {
		display: flex;
	}