
	.block.svelte-1plpy97 {
		position: relative;
		margin: 0;
		box-shadow: var(--block-shadow);
		border-width: var(--block-border-width);
		border-color: var(--block-border-color);
		border-radius: var(--block-radius);
		background: var(--block-background-fill);
		width: 100%;
		line-height: var(--line-sm);
		border-width: var(--block-border-width);
	}
	.block.fullscreen.svelte-1plpy97 {
		border-radius: 0;
	}

	.auto-margin.svelte-1plpy97 {
		margin-left: auto;
		margin-right: auto;
	}

	.block.border_focus.svelte-1plpy97 {
		border-color: var(--color-accent);
	}

	.block.border_contrast.svelte-1plpy97 {
		border-color: var(--body-text-color);
	}

	.padded.svelte-1plpy97 {
		padding: var(--block-padding);
	}

	.hidden.svelte-1plpy97 {
		/* We use "hidden" class name in form/Index.svelte to hide parent forms, so if this class changes, edit form/Index.svelte accordingly */
		display: none;
	}

	.flex.svelte-1plpy97 {
		display: flex;
		flex-direction: column;
	}
	.hide-container.svelte-1plpy97:not(.fullscreen) {
		margin: 0;
		box-shadow: none;
		border-width: 0;
		background: transparent;
		padding: 0;
		overflow: visible;
	}
	.resize-handle.svelte-1plpy97 {
		position: absolute;
		bottom: 0;
		right: 0;
		width: 10px;
		height: 10px;
		fill: var(--block-border-color);
		cursor: nwse-resize;
	}
	.fullscreen.svelte-1plpy97 {
		position: fixed;
		top: 0;
		left: 0;
		width: 100vw;
		height: 100vh;
		z-index: 1000;
		overflow: auto;
	}

	.animating.svelte-1plpy97 {
		animation: svelte-1plpy97-pop-out 0.1s ease-out forwards;
	}

	@keyframes svelte-1plpy97-pop-out {
		0% {
			position: fixed;
			top: var(--start-top);
			left: var(--start-left);
			width: var(--start-width);
			height: var(--start-height);
			z-index: 100;
		}
		100% {
			position: fixed;
			top: 0vh;
			left: 0vw;
			width: 100vw;
			height: 100vh;
			z-index: 1000;
		}
	}

	.placeholder.svelte-1plpy97 {
		border-radius: var(--block-radius);
		border-width: var(--block-border-width);
		border-color: var(--block-border-color);
		border-style: dashed;
	}
	.info-text.svelte-9hc4ua {
		font-weight: var(--block-info-text-weight);
		font-size: var(--block-info-text-size);
		line-height: var(--line-sm);
		color: var(--block-info-text-color);
		margin-bottom: var(--spacing-md);
	}
	.info-text.svelte-9hc4ua a {
		color: var(--link-text-color);
		text-decoration: underline;
	}
	.info-text.svelte-9hc4ua code {
		background: var(--code-background-fill);
		padding: 0.1em 0.3em;
		border-radius: 3px;
		font-size: 0.9em;
	}
	span.has-info.svelte-jdcl7l {
		margin-bottom: var(--spacing-xs);
	}
	span.svelte-jdcl7l:not(.has-info) {
		margin-bottom: var(--spacing-lg);
	}
	span.svelte-jdcl7l {
		display: inline-block;
		position: relative;
		z-index: var(--layer-4);
		border: solid var(--block-title-border-width)
			var(--block-title-border-color);
		border-radius: var(--block-title-radius);
		background: var(--block-title-background-fill);
		padding: var(--block-title-padding);
		color: var(--block-title-text-color);
		font-weight: var(--block-title-text-weight);
		font-size: var(--block-title-text-size);
		line-height: var(--line-sm);
	}

	span[dir="rtl"].svelte-jdcl7l {
		display: block;
	}

	.hide.svelte-jdcl7l {
		margin: 0;
		height: 0;
	}

	.sr-only.svelte-jdcl7l {
		clip: rect(0, 0, 0, 0);
		position: absolute;
		margin: -1px;
		border-width: 0;
		padding: 0;
		width: 1px;
		height: 1px;
		overflow: hidden;
		white-space: nowrap;
	}
	label.svelte-19djge9 {
		display: inline-flex;
		align-items: center;
		z-index: var(--layer-2);
		box-shadow: var(--block-label-shadow);
		border: var(--block-label-border-width) solid
			var(--block-label-border-color);
		border-top: none;
		border-left: none;
		border-radius: var(--block-label-radius);
		background: var(--block-label-background-fill);
		padding: var(--block-label-padding);
		pointer-events: none;
		color: var(--block-label-text-color);
		font-weight: var(--block-label-text-weight);
		font-size: var(--block-label-text-size);
		line-height: var(--line-sm);
	}
	.gr-group label.svelte-19djge9 {
		border-top-left-radius: 0;
	}

	label.float.svelte-19djge9 {
		position: absolute;
		top: var(--block-label-margin);
		left: var(--block-label-margin);
	}
	label.svelte-19djge9:not(.float) {
		position: static;
		margin-top: var(--block-label-margin);
		margin-left: var(--block-label-margin);
	}

	.hide.svelte-19djge9 {
		display: none;
	}

	span.svelte-19djge9 {
		opacity: 0.8;
		margin-right: var(--size-2);
		width: calc(var(--block-label-text-size) - 1px);
		height: calc(var(--block-label-text-size) - 1px);
	}
	.hide-label.svelte-19djge9 {
		box-shadow: none;
		border-width: 0;
		background: transparent;
		overflow: visible;
	}

	label[dir="rtl"].svelte-19djge9 {
		border: var(--block-label-border-width) solid
			var(--block-label-border-color);
		border-top: none;
		border-right: none;
		border-bottom-left-radius: var(--block-radius);
		border-bottom-right-radius: var(--block-label-radius);
		border-top-left-radius: var(--block-label-radius);
	}

	label[dir="rtl"].svelte-19djge9 span:where(.svelte-19djge9) {
		margin-left: var(--size-2);
		margin-right: 0;
	}
	.unstyled-link.svelte-7nkusa {
		all: unset;
		cursor: pointer;
	}
	button.svelte-3jwzs9 {
		display: flex;
		justify-content: center;
		align-items: center;
		gap: 1px;
		z-index: var(--layer-2);
		border-radius: var(--radius-xs);
		color: var(--block-label-text-color);
		border: 1px solid var(--border-color);
		padding: var(--spacing-xxs);
	}

	button.svelte-3jwzs9:hover {
		background-color: var(--background-fill-secondary);
	}

	button[disabled].svelte-3jwzs9 {
		opacity: 0.5;
		box-shadow: none;
	}

	button[disabled].svelte-3jwzs9:hover {
		cursor: not-allowed;
	}

	.padded.svelte-3jwzs9 {
		background: var(--bg-color);
	}

	button.svelte-3jwzs9:hover,
	button.highlight.svelte-3jwzs9 {
		cursor: pointer;
		color: var(--color-accent);
	}

	.padded.svelte-3jwzs9:hover {
		color: var(--block-label-text-color);
	}

	span.svelte-3jwzs9 {
		padding: 0px 1px;
		font-size: 10px;
	}

	div.svelte-3jwzs9 {
		display: flex;
		align-items: center;
		justify-content: center;
		transition: filter 0.2s ease-in-out;
	}

	.x-small.svelte-3jwzs9 {
		width: 10px;
		height: 10px;
	}

	.small.svelte-3jwzs9 {
		width: 14px;
		height: 14px;
	}

	.medium.svelte-3jwzs9 {
		width: 20px;
		height: 20px;
	}

	.large.svelte-3jwzs9 {
		width: 22px;
		height: 22px;
	}

	.pending.svelte-3jwzs9 {
		animation: svelte-3jwzs9-flash 0.5s infinite;
	}

	@keyframes svelte-3jwzs9-flash {
		0% {
			opacity: 0.5;
		}
		50% {
			opacity: 1;
		}
		100% {
			opacity: 0.5;
		}
	}

	.transparent.svelte-3jwzs9 {
		background: transparent;
		border: none;
		box-shadow: none;
	}
	.empty.svelte-v95lt3 {
		display: flex;
		justify-content: center;
		align-items: center;
		margin-top: calc(0px - var(--size-6));
		height: var(--size-full);
	}

	.icon.svelte-v95lt3 {
		opacity: 0.5;
		height: var(--size-5);
		color: var(--body-text-color);
	}

	.small.svelte-v95lt3 {
		min-height: calc(var(--size-32) - 20px);
	}

	.large.svelte-v95lt3 {
		min-height: calc(var(--size-64) - 20px);
	}

	.unpadded_box.svelte-v95lt3 {
		margin-top: 0;
	}

	.small_parent.svelte-v95lt3 {
		min-height: 100% !important;
	}
	.dropdown-arrow.svelte-loyhyk {
		fill: currentColor;
		/* margin-right: var(--size-2); */
		/* width: var(--size-5); */
	}
	.dropdown-arrow.svelte-1w1fnc7 {
		fill: currentColor;
	}

	.circle.svelte-1w1fnc7 {
		fill: currentColor;
		opacity: 0.1;
	}
	svg.svelte-l5a973 {
		animation: svelte-l5a973-spin 1.5s linear infinite;
	}

	@keyframes svelte-l5a973-spin {
		0% {
			transform: rotate(0deg);
		}
		100% {
			transform: rotate(360deg);
		}
	}
	h2.svelte-1vmd51o {
		font-size: var(--text-xl) !important;
	}

	p.svelte-1vmd51o,
	h2.svelte-1vmd51o {
		white-space: pre-line;
	}

	.wrap.svelte-1vmd51o {
		display: flex;
		flex-direction: column;
		justify-content: center;
		align-items: center;
		min-height: var(--size-60);
		color: var(--block-label-text-color);
		line-height: var(--line-md);
		height: 100%;
		padding-top: var(--size-3);
		text-align: center;
		margin: auto var(--spacing-lg);
	}

	.or.svelte-1vmd51o {
		color: var(--body-text-color-subdued);
		display: flex;
	}

	.icon-wrap.svelte-1vmd51o {
		width: 30px;
		margin-bottom: var(--spacing-lg);
	}

	@media (min-width: 768px) {
		.wrap.svelte-1vmd51o {
			font-size: var(--text-lg);
		}
	}

	.hovered.svelte-1vmd51o {
		color: var(--color-accent);
	}
	.source-selection.svelte-exvkcd {
		display: flex;
		align-items: center;
		justify-content: center;
		border-top: 1px solid var(--border-color-primary);
		width: 100%;
		margin-left: auto;
		margin-right: auto;
		height: var(--size-10);
	}

	.icon.svelte-exvkcd {
		width: 22px;
		height: 22px;
		margin: var(--spacing-lg) var(--spacing-xs);
		padding: var(--spacing-xs);
		color: var(--neutral-400);
		border-radius: var(--radius-md);
	}

	.selected.svelte-exvkcd {
		color: var(--color-accent);
	}

	.icon.svelte-exvkcd:hover,
	.icon.svelte-exvkcd:focus {
		color: var(--color-accent);
	}
	.custom-button.svelte-gnx6f7 {
		display: flex;
		align-items: center;
		justify-content: center;
		gap: var(--size-1);
		height: var(--size-6);
		padding: var(--size-1) var(--size-2);
		border: none;
		border-radius: var(--radius-sm);
		background: transparent;
		color: var(--block-title-text-color);
		cursor: pointer;
		transition: all 0.2s;
		font-size: var(--text-sm);
		margin-left: var(--size-1);
	}

	.custom-button.svelte-gnx6f7:hover {
		background: var(--background-fill-secondary);
		color: var(--body-text-color);
	}

	.custom-button-label.svelte-gnx6f7 {
		white-space: nowrap;
	}
	.icon-button-wrapper.svelte-1pnho82 {
		display: flex;
		flex-direction: row;
		align-items: center;
		justify-content: center;
		z-index: var(--layer-2);
		gap: var(--spacing-sm);
		box-shadow: var(--shadow-drop);
		border: 1px solid var(--border-color-primary);
		background: var(--block-background-fill);
		padding: var(--spacing-xxs);
	}

	.icon-button-wrapper.hide-top-corner.svelte-1pnho82 {
		border-top: none;
		border-right: none;
		border-radius: var(--block-label-right-radius);
	}

	.icon-button-wrapper.display-top-corner.svelte-1pnho82 {
		border-radius: var(--radius-sm) 0 0 var(--radius-sm);
		top: var(--spacing-sm);
		right: -1px;
	}

	.icon-button-wrapper.svelte-1pnho82:not(.top-panel) {
		border: 1px solid var(--border-color-primary);
		border-radius: var(--radius-sm);
	}

	.top-panel.svelte-1pnho82 {
		position: absolute;
		top: var(--block-label-margin);
		right: var(--block-label-margin);
		margin: 0;
	}

	.icon-button-wrapper.svelte-1pnho82 button {
		margin: var(--spacing-xxs);
		border-radius: var(--radius-xs);
		position: relative;
	}

	.icon-button-wrapper.svelte-1pnho82 a.download-link:not(:last-child),
	.icon-button-wrapper.svelte-1pnho82 button:not(:last-child) {
		margin-right: var(--spacing-xxs);
	}

	.icon-button-wrapper.svelte-1pnho82
		a.download-link:not(:last-child):not(.no-border *)::after,
	.icon-button-wrapper.svelte-1pnho82
		button:not(:last-child):not(.no-border *)::after {
		content: "";
		position: absolute;
		right: -4.5px;
		top: 15%;
		height: 70%;
		width: 1px;
		background-color: var(--border-color-primary);
	}

	.icon-button-wrapper.svelte-1pnho82 > * {
		height: 100%;
	}

	.icon-button-wrapper.no-background.svelte-1pnho82 {
		box-shadow: none;
		border: none;
		background: none;
		padding: 0;
		z-index: var(--layer-1);
	}
	.scroll-fade.svelte-kmbucf {
		position: sticky;
		bottom: calc(0px - var(--size-2-5));
		left: 0;
		right: 0;
		height: var(--size-6);
		background: linear-gradient(
			to top,
			var(--block-background-fill) 30%,
			transparent
		);
		pointer-events: none;
		margin-top: calc(0px - var(--size-6));
	}

	.scroll-fade.absolute.svelte-kmbucf {
		position: absolute;
		bottom: 0;
		margin-top: 0;
		z-index: var(--layer-2);
	}