declare namespace svelteHTML {
	interface HTMLAttributes<T extends EventTarget> {
		webkitdirectory?: boolean | string;
		mozdirectory?: boolean | string;
	}
}
declare module "@lezer/highlight";
declare module "dequal/lite";
declare module "dequal";
