
	.file-name.svelte-ey25pz {
		padding: var(--size-6);
		font-size: var(--text-xxl);
		word-break: break-all;
	}

	.file-size.svelte-ey25pz {
		padding: var(--size-2);
		font-size: var(--text-xl);
	}

	.upload-container.svelte-ey25pz {
		height: 100%;
		width: 100%;
	}

	.video-container.svelte-ey25pz {
		display: flex;
		height: 100%;
		flex-direction: column;
		justify-content: center;
		align-items: center;
	}