
	div.svelte-19utvcn {
		width: 100%;
		height: 100%;
	}
	#vg-tooltip-element {
		font-family: var(--font) !important;
		font-size: var(--text-xs) !important;
		box-shadow: none !important;
		background-color: var(--block-background-fill) !important;
		border: 1px solid var(--border-color-primary) !important;
		color: var(--body-text-color) !important;
	}
	#vg-tooltip-element .key {
		color: var(--body-text-color-subdued) !important;
	}
	.caption.svelte-19utvcn {
		padding: 0 4px;
		margin: 0;
		text-align: center;
	}