const index = 1;
let component_cache;
const component = async () => component_cache ??= (await import('./error.svelte-BKTK9npS.js')).default;
const imports = ["_app/immutable/nodes/1.C3ecUjVm.js","_app/immutable/chunks/BEYfkQLb.js","_app/immutable/chunks/DcfTJGrh.js","_app/immutable/chunks/DE3DmrRd.js","_app/immutable/chunks/C903eyvM.js","_app/immutable/chunks/DA4iNbiz.js","_app/immutable/chunks/1t7wABRY.js","_app/immutable/chunks/43y08qZT.js"];
const stylesheets = [];
const fonts = [];

export { component, fonts, imports, index, stylesheets };
//# sourceMappingURL=1-40J_SSlg.js.map
