.gradio-container-6-11-0 Tables */ table,
.gradio-container-6-11-0 tr,
.gradio-container-6-11-0 td,
.gradio-container-6-11-0 th {
	margin-top: var(--spacing-sm);
	margin-bottom: var(--spacing-sm);
	padding: var(--spacing-xl);
}

/* .message-wrap :global(pre[class*="language-"]),
	.message-wrap :global(pre) {
		border: none;
		background: none;
		position: relative;
		direction: ltr;
		white-space: no-wrap;
		overflow-x: auto;
	}
	.message-wrap :global(code) {
	} */

/* .message-wrap :global(div[class*="code_wrap"]) {
		
	} */

.gradio-container-6-11-0 .md code,
.gradio-container-6-11-0 .md pre {
	background: none;
	font-family: var(--font-mono);
	font-size: var(--text-sm);

	text-align: left;
	white-space: pre;
	word-spacing: normal;
	word-break: normal;
	word-wrap: normal;
	line-height: 1.5;
	tab-size: 2;

	hyphens: none;
}

.gradio-container-6-11-0 .md pre[class*="language-"]::-moz-selection,
.gradio-container-6-11-0 .md pre[class*="language-"] ::-moz-selection,
.gradio-container-6-11-0 .md code[class*="language-"]::-moz-selection,
.gradio-container-6-11-0 .md code[class*="language-"] ::-moz-selection {
}

.gradio-container-6-11-0 .md pre[class*="language-"]::selection,
.gradio-container-6-11-0 .md pre[class*="language-"] ::selection,
.gradio-container-6-11-0 .md code[class*="language-"]::selection,
.gradio-container-6-11-0 .md code[class*="language-"] ::selection {
	text-shadow: none;
	background: #b3d4fc;
}

/* Code blocks */
.gradio-container-6-11-0 .md pre {
	padding: 1em;
	margin: 0.5em 0;
	overflow: auto;
	position: relative;
	margin-top: var(--spacing-sm);
	margin-bottom: var(--spacing-sm);
	box-shadow: none;
	border: none;
	border-radius: var(--radius-md);
	background: var(--code-background-fill);
	padding: var(--spacing-xxl);
	font-family: var(--font-mono);
	/* font-size: var(--text-sm) !important; */
	display: block;
	white-space: pre;
	border-radius: var(--radius-sm);
	text-shadow: none;
	border-radius: var(--radius-sm);
	/* font-size: 85%; */
	white-space: nowrap;
	display: block;
	white-space: pre;
}

.gradio-container-6-11-0 .prose code {
}
.gradio-container-6-11-0 .prose pre > code {
}

/* Inline code */
.gradio-container-6-11-0 .md :not(pre) > code {
	padding: 0.1em;
	border-radius: var(--radius-xs);
	white-space: normal;
	background: var(--code-background-fill);
	border: 1px solid var(--panel-border-color);
	padding: var(--spacing-xxs) var(--spacing-xs);
}

.gradio-container-6-11-0 .md .token.comment,
.gradio-container-6-11-0 .md .token.prolog,
.gradio-container-6-11-0 .md .token.doctype,
.gradio-container-6-11-0 .md .token.cdata {
	color: slategray;
}

.gradio-container-6-11-0 .md .token.punctuation {
	color: #999;
}

.gradio-container-6-11-0 .md .token.namespace {
	opacity: 0.7;
}

.gradio-container-6-11-0 .md .token.property,
.gradio-container-6-11-0 .md .token.tag,
.gradio-container-6-11-0 .md .token.boolean,
.gradio-container-6-11-0 .md .token.number,
.gradio-container-6-11-0 .md .token.constant,
.gradio-container-6-11-0 .md .token.symbol,
.gradio-container-6-11-0 .md .token.deleted {
	color: #905;
}

.gradio-container-6-11-0 .md .token.selector,
.gradio-container-6-11-0 .md .token.attr-name,
.gradio-container-6-11-0 .md .token.string,
.gradio-container-6-11-0 .md .token.char,
.gradio-container-6-11-0 .md .token.builtin,
.gradio-container-6-11-0 .md .token.inserted {
	color: #690;
}

.gradio-container-6-11-0 .md .token.atrule,
.gradio-container-6-11-0 .md .token.attr-value,
.gradio-container-6-11-0 .md .token.keyword {
	color: #07a;
}

.gradio-container-6-11-0 .md .token.function,
.gradio-container-6-11-0 .md .token.class-name {
	color: #dd4a68;
}

.gradio-container-6-11-0 .md .token.regex,
.gradio-container-6-11-0 .md .token.important,
.gradio-container-6-11-0 .md .token.variable {
	color: #e90;
}

.gradio-container-6-11-0 .md .token.important,
.gradio-container-6-11-0 .md .token.bold {
	font-weight: bold;
}
.gradio-container-6-11-0 .md .token.italic {
	font-style: italic;
}

.gradio-container-6-11-0 .md .token.entity {
	cursor: help;
}

.dark .md .token.comment,
.dark .md .token.prolog,
.dark .md .token.cdata {
	color: hsl(220, 10%, 40%);
}

.dark .md .token.doctype,
.dark .md .token.punctuation,
.dark .md .token.entity {
	color: hsl(220, 14%, 71%);
}

.dark .md .token.attr-name,
.dark .md .token.class-name,
.dark .md .token.boolean,
.dark .md .token.constant,
.dark .md .token.number,
.dark .md .token.atrule {
	color: hsl(29, 54%, 61%);
}

.dark .md .token.keyword {
	color: hsl(286, 60%, 67%);
}

.dark .md .token.property,
.dark .md .token.tag,
.dark .md .token.symbol,
.dark .md .token.deleted,
.dark .md .token.important {
	color: hsl(355, 65%, 65%);
}

.dark .md .token.selector,
.dark .md .token.string,
.dark .md .token.char,
.dark .md .token.builtin,
.dark .md .token.inserted,
.dark .md .token.regex,
.dark .md .token.attr-value,
.dark .md .token.attr-value > .token.punctuation {
	color: hsl(95, 38%, 62%);
}

.dark .md .token.variable,
.dark .md .token.operator,
.dark .md .token.function {
	color: hsl(207, 82%, 66%);
}

.dark .md .token.url {
	color: hsl(187, 47%, 55%);
}

	span.svelte-1hf8a14 div[class*="code_wrap"] {
		position: relative;
	}

	/* KaTeX */
	span.svelte-1hf8a14 span.katex {
		font-size: var(--text-lg);
		direction: ltr;
	}

	span.svelte-1hf8a14 div[class*="code_wrap"] > button {
		z-index: 1;
		cursor: pointer;
		border-bottom-left-radius: var(--radius-sm);
		padding: var(--spacing-md);
		width: 25px;
		height: 25px;
		position: absolute;
		right: 0;
	}

	span.svelte-1hf8a14 .check {
		opacity: 0;
		z-index: var(--layer-top);
		transition: opacity 0.2s;
		background: var(--code-background-fill);
		color: var(--body-text-color);
		position: absolute;
		top: var(--size-1-5);
		left: var(--size-1-5);
	}

	span.svelte-1hf8a14 p:not(:first-child) {
		margin-top: var(--spacing-xxl);
	}

	span.svelte-1hf8a14 .md-header-anchor {
		/* position: absolute; */
		margin-left: -25px;
		padding-right: 8px;
		line-height: 1;
		color: var(--body-text-color-subdued);
		opacity: 0;
	}

	span.svelte-1hf8a14 h1:hover .md-header-anchor,
	span.svelte-1hf8a14 h2:hover .md-header-anchor,
	span.svelte-1hf8a14 h3:hover .md-header-anchor,
	span.svelte-1hf8a14 h4:hover .md-header-anchor,
	span.svelte-1hf8a14 h5:hover .md-header-anchor,
	span.svelte-1hf8a14 h6:hover .md-header-anchor {
		opacity: 1;
	}

	span.md.svelte-1hf8a14 .md-header-anchor > svg {
		color: var(--body-text-color-subdued);
	}

	span.svelte-1hf8a14 table {
		word-break: break-word;
	}