import './i18n-CMo5lpzy.js';
import { a as append, f as from_svg } from './index-BvBk1Iap.js';

var root = from_svg(`<svg width="100%" height="100%" viewBox="0 0 24 24" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" xml:space="preserve" stroke="currentColor" style="fill-rule:evenodd;clip-rule:evenodd;stroke-linecap:round;stroke-linejoin:round;"><g transform="matrix(1.14096,-0.140958,-0.140958,1.14096,-0.0559523,0.0559523)"><path d="M18,6L6.087,17.913" style="fill:none;fill-rule:nonzero;stroke-width:2px;"></path></g><path d="M4.364,4.364L19.636,19.636" style="fill:none;fill-rule:nonzero;stroke-width:2px;"></path></svg>`);

function Clear($$anchor) {
	var svg = root();

	append($$anchor, svg);
}

export { Clear as C };
//# sourceMappingURL=Clear-Dgs5UhTO.js.map
