import './i18n-CMo5lpzy.js';
import { a as append, f as from_svg } from './index-BvBk1Iap.js';

var root = from_svg(`<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="feather feather-maximize" width="100%" height="100%"><path d="M8 3H5a2 2 0 0 0-2 2v3m18 0V5a2 2 0 0 0-2-2h-3m0 18h3a2 2 0 0 0 2-2v-3M3 16v3a2 2 0 0 0 2 2h3"></path></svg>`);

function Maximize($$anchor) {
	var svg = root();

	append($$anchor, svg);
}

export { Maximize as M };
//# sourceMappingURL=Maximize-B_43ylTs.js.map
