
	.label-clear-button.svelte-al0bnp {
		color: var(--body-text-color-subdued);
		position: relative;
		left: -3px;
	}

	.label-clear-button.svelte-al0bnp:hover {
		color: var(--body-text-color);
	}

	.file-preview.svelte-al0bnp {
		table-layout: fixed;
		width: var(--size-full);
		max-height: var(--size-60);
		overflow-y: auto;
		margin-top: var(--size-1);
		color: var(--body-text-color);
	}

	.file-preview-holder.svelte-al0bnp {
		overflow: auto;
	}

	.file.svelte-al0bnp {
		display: flex;
		width: var(--size-full);
	}

	.file.svelte-al0bnp > :where(.svelte-al0bnp) {
		padding: var(--size-1) var(--size-2-5);
	}

	.filename.svelte-al0bnp {
		flex-grow: 1;
		display: flex;
		overflow: hidden;
	}
	.filename.svelte-al0bnp .stem:where(.svelte-al0bnp) {
		overflow: hidden;
		text-overflow: ellipsis;
		white-space: nowrap;
	}
	.filename.svelte-al0bnp .ext:where(.svelte-al0bnp) {
		white-space: nowrap;
	}

	.download.svelte-al0bnp {
		min-width: 8rem;
		width: 10%;
		white-space: nowrap;
		text-align: right;
	}
	.download.svelte-al0bnp:hover {
		text-decoration: underline;
	}
	.download.svelte-al0bnp > a {
		color: var(--link-text-color);
	}

	.download.svelte-al0bnp > a:hover {
		color: var(--link-text-color-hover);
	}
	.download.svelte-al0bnp > a:visited {
		color: var(--link-text-color-visited);
	}
	.download.svelte-al0bnp > a:active {
		color: var(--link-text-color-active);
	}
	.selectable.svelte-al0bnp {
		cursor: pointer;
	}

	tbody.svelte-al0bnp > tr:where(.svelte-al0bnp):nth-child(even) {
		background: var(--block-background-fill);
	}

	tbody.svelte-al0bnp > tr:where(.svelte-al0bnp):nth-child(odd) {
		background: var(--table-odd-background-fill);
	}

	.drag-handle.svelte-al0bnp {
		cursor: grab;
		color: var(--body-text-color-subdued);
		padding-right: var(--size-2);
		user-select: none;
	}

	.dragging.svelte-al0bnp {
		opacity: 0.5;
		cursor: grabbing;
	}

	.drop-target.svelte-al0bnp {
		border-top: 2px solid var(--color-accent);
	}

	tr:last-child.drop-target[data-drop-target="before"].svelte-al0bnp {
		border-top: 2px solid var(--color-accent);
		border-bottom: none;
	}

	tr:last-child.drop-target[data-drop-target="after"].svelte-al0bnp {
		border-top: none;
		border-bottom: 2px solid var(--color-accent);
	}