export const redirects = {
	"/guides/creating-a-new-component":
		"/guides/custom-components-in-five-minutes",
	"/guides/five-minute-guide": "/guides/custom-components-in-five-minutes",
	"/guides/key-features": "/guides/queuing",
	"/quickstart": "/guides/quickstart",
	"/sharing-your-app": "/guides/sharing-your-app",
	"/introduction_to_blocks": "/guides/blocks-and-event-listeners",
	"/getting_started": "/guides/quickstart",
	"/guides/building-with-blocks": "/guides/blocks-and-event-listeners",
	"/demos": "/docs",
	"/playground": "/docs",
	"/docs/client": "/docs/python-client/client",
	"/docs/blocks": "/docs/gradio/blocks",
	"/docs/audio": "/docs/gradio/audio",
	"/docs/image": "/docs/gradio/image",
	"/docs/textbox": "/docs/gradio/textbox",
	"/docs/interface": "/docs/gradio/interface",
	"/docs/python-client/python-client": "/docs/python-client/introduction",
	"/guides/sharing-your-app#security-and-file-access": "/guides/file-access",
	"/docs/gradio/interface#interface-queue": "/docs/gradio/interface",
	"/using_hugging_face_integrations": "/guides/using-hugging-face-integrations",
	"/main/guides/object-detection-from-webcam":
		"/main/guides/object-detection-from-webcam-with-webrtc",
	"/guides/chatbots/chat_interface_examples":
		"/main/guides/chatinterface-examples",
	"/guides/chatbots/creating-a-discord-bot-from-a-gradio-app":
		"/guides/creating-a-discord-bot-from-a-gradio-app",
	"/guides/chatbots/agents-and-tool-usage": "/guides/agents-and-tool-usage"
};
