import './i18n-CMo5lpzy.js';
import { a7 as text, t as template_effect, a as append, a0 as set_text, a8 as next } from './index-BvBk1Iap.js';

function Example($$anchor, $$props) {
	next();

	var text$1 = text();

	template_effect(() => set_text(text$1, $$props.value || ""));
	append($$anchor, text$1);
}

export { Example as default };
//# sourceMappingURL=Example-BA64u5Sp.js.map
