.gradio-container-6-11-0 Tables */ table,
.gradio-container-6-11-0 tr,
.gradio-container-6-11-0 td,
.gradio-container-6-11-0 th {
	margin-top: var(--spacing-sm);
	margin-bottom: var(--spacing-sm);
	padding: var(--spacing-xl);
}

/* .message-wrap :global(pre[class*="language-"]),
	.message-wrap :global(pre) {
		border: none;
		background: none;
		position: relative;
		direction: ltr;
		white-space: no-wrap;
		overflow-x: auto;
	}
	.message-wrap :global(code) {
	} */

/* .message-wrap :global(div[class*="code_wrap"]) {
		
	} */

.gradio-container-6-11-0 .md code,
.gradio-container-6-11-0 .md pre {
	background: none;
	font-family: var(--font-mono);
	font-size: var(--text-md);

	text-align: left;
	white-space: pre;
	word-spacing: normal;
	word-break: normal;
	word-wrap: normal;
	line-height: 1.5;
	tab-size: 2;

	hyphens: none;
}

.gradio-container-6-11-0 .md pre[class*="language-"]::-moz-selection,
.gradio-container-6-11-0 .md pre[class*="language-"] ::-moz-selection,
.gradio-container-6-11-0 .md code[class*="language-"]::-moz-selection,
.gradio-container-6-11-0 .md code[class*="language-"] ::-moz-selection {
}

.gradio-container-6-11-0 .md pre[class*="language-"]::selection,
.gradio-container-6-11-0 .md pre[class*="language-"] ::selection,
.gradio-container-6-11-0 .md code[class*="language-"]::selection,
.gradio-container-6-11-0 .md code[class*="language-"] ::selection {
	text-shadow: none;
	background: #b3d4fc;
}

/* Code blocks */
.gradio-container-6-11-0 .md pre {
	padding: 1em;
	margin: 0.5em 0;
	overflow: auto;
	position: relative;
	margin-top: var(--spacing-sm);
	margin-bottom: var(--spacing-sm);
	box-shadow: none;
	border: none;
	border-radius: var(--radius-md);
	background: var(--code-background-fill);
	font-family: var(--font-mono);
	display: block;
	white-space: pre;
	border-radius: var(--radius-sm);
	text-shadow: none;
	border-radius: var(--radius-sm);
	/* font-size: 85%; */
	white-space: nowrap;
	display: block;
	white-space: pre;
}

.gradio-container-6-11-0 .prose code {
}
.gradio-container-6-11-0 .prose pre > code {
}

/* Inline code */
.gradio-container-6-11-0 .md :not(pre) > code {
	padding: 0.1em;
	border-radius: var(--radius-xs);
	white-space: normal;
	background: var(--code-background-fill);
	border: 1px solid var(--panel-border-color);
	padding: var(--spacing-xxs) var(--spacing-xs);
}

.gradio-container-6-11-0 .md .token.comment,
.gradio-container-6-11-0 .md .token.prolog,
.gradio-container-6-11-0 .md .token.doctype,
.gradio-container-6-11-0 .md .token.cdata {
	color: slategray;
}

.gradio-container-6-11-0 .md .token.punctuation {
	color: #999;
}

.gradio-container-6-11-0 .md .token.namespace {
	opacity: 0.7;
}

.gradio-container-6-11-0 .md .token.property,
.gradio-container-6-11-0 .md .token.tag,
.gradio-container-6-11-0 .md .token.boolean,
.gradio-container-6-11-0 .md .token.number,
.gradio-container-6-11-0 .md .token.constant,
.gradio-container-6-11-0 .md .token.symbol,
.gradio-container-6-11-0 .md .token.deleted {
	color: #905;
}

.gradio-container-6-11-0 .md .token.selector,
.gradio-container-6-11-0 .md .token.attr-name,
.gradio-container-6-11-0 .md .token.string,
.gradio-container-6-11-0 .md .token.char,
.gradio-container-6-11-0 .md .token.builtin,
.gradio-container-6-11-0 .md .token.inserted {
	color: #690;
}

.gradio-container-6-11-0 .md .token.atrule,
.gradio-container-6-11-0 .md .token.attr-value,
.gradio-container-6-11-0 .md .token.keyword {
	color: #07a;
}

.gradio-container-6-11-0 .md .token.function,
.gradio-container-6-11-0 .md .token.class-name {
	color: #dd4a68;
}

.gradio-container-6-11-0 .md .token.regex,
.gradio-container-6-11-0 .md .token.important,
.gradio-container-6-11-0 .md .token.variable {
	color: #e90;
}

.gradio-container-6-11-0 .md .token.important,
.gradio-container-6-11-0 .md .token.bold {
	font-weight: bold;
}
.gradio-container-6-11-0 .md .token.italic {
	font-style: italic;
}

.gradio-container-6-11-0 .md .token.entity {
	cursor: help;
}

.dark .md .token.comment,
.dark .md .token.prolog,
.dark .md .token.cdata {
	color: hsl(220, 10%, 40%);
}

.dark .md .token.doctype,
.dark .md .token.punctuation,
.dark .md .token.entity {
	color: hsl(220, 14%, 71%);
}

.dark .md .token.attr-name,
.dark .md .token.class-name,
.dark .md .token.boolean,
.dark .md .token.constant,
.dark .md .token.number,
.dark .md .token.atrule {
	color: hsl(29, 54%, 61%);
}

.dark .md .token.keyword {
	color: hsl(286, 60%, 67%);
}

.dark .md .token.property,
.dark .md .token.tag,
.dark .md .token.symbol,
.dark .md .token.deleted,
.dark .md .token.important {
	color: hsl(355, 65%, 65%);
}

.dark .md .token.selector,
.dark .md .token.string,
.dark .md .token.char,
.dark .md .token.builtin,
.dark .md .token.inserted,
.dark .md .token.regex,
.dark .md .token.attr-value,
.dark .md .token.attr-value > .token.punctuation {
	color: hsl(95, 38%, 62%);
}

.dark .md .token.variable,
.dark .md .token.operator,
.dark .md .token.function {
	color: hsl(207, 82%, 66%);
}

.dark .md .token.url {
	color: hsl(187, 47%, 55%);
}

	.header.svelte-1kuiw39 {
		display: flex;
		justify-content: space-between;
		align-items: center;
		padding: 0.7rem 1rem;
		border-bottom: 1px solid var(--table-border-color);
	}

	.title.svelte-1kuiw39 {
		font-size: var(--scale-0);
		font-weight: 600;
		color: var(--body-text-color);
	}

	.toggle-all.svelte-1kuiw39 {
		background: none;
		border: none;
		cursor: pointer;
		padding: 0;
		color: var(--body-text-color);
		font-size: 0.7em;
		line-height: 1;
		opacity: 0.7;
		transition:
			opacity 0.2s ease,
			transform 0.3s ease;
	}

	.toggle-all.svelte-1kuiw39:hover {
		opacity: 1;
	}

	.wrap[data-all-open="true"] .toggle-all.svelte-1kuiw39 {
		transform: rotate(180deg);
	}

	.default.svelte-1kuiw39 pre,
	.default.svelte-1kuiw39 .highlight {
		display: inline-block;
	}

	.wrap.svelte-1kuiw39 pre,
	.wrap.svelte-1kuiw39 .highlight {
		margin: 0 !important;
		background: transparent !important;
		font-family: var(--font-mono);
		font-weight: 400;
		padding: 0 !important;
	}

	.wrap.svelte-1kuiw39 pre a {
		color: var(--link-text-color-hover);
		text-decoration: underline;
	}

	.wrap.svelte-1kuiw39 pre a:hover {
		color: var(--link-text-color-hover);
	}

	.default.svelte-1kuiw39 > span:where(.svelte-1kuiw39) {
		text-transform: uppercase;
		font-size: 0.7rem;
		font-weight: 600;
	}

	.default.svelte-1kuiw39 > code:where(.svelte-1kuiw39) {
		border: none;
	}
	code.svelte-1kuiw39 {
		background: none;
		font-family: var(--font-mono);
	}

	.wrap.svelte-1kuiw39 {
		padding: 0rem;
		border-radius: 5px;
		overflow: hidden;
		position: relative;
		margin: 0;
		box-shadow: var(--block-shadow);
		border-width: var(--block-border-width);
		border-color: var(--block-border-color);
		border-radius: var(--block-radius);
		width: 100%;
		line-height: var(--line-sm);
		color: var(--body-text-color);
		display: grid;
		grid-template-rows: auto 1fr;
	}

	.type.svelte-1kuiw39 {
		position: relative;
		padding: 0.7rem 1rem;
		padding-left: 2rem;
		background: var(--table-odd-background-fill);
		border-bottom: 0px solid var(--table-border-color);
		list-style: none;
	}

	.type.svelte-1kuiw39::after {
		content: "▼";
		position: absolute;
		top: 50%;
		right: 15px;
		transform: translateY(-50%);
		transition: transform 0.3s ease;
		font-size: 0.7em;
		opacity: 0.7;
	}

	details[open].svelte-1kuiw39 .type:where(.svelte-1kuiw39)::after {
		transform: translateY(-50%) rotate(180deg);
	}

	.default.svelte-1kuiw39 {
		padding: 0.2rem 1rem 0.3rem 1rem;
		border-bottom: 1px solid var(--table-border-color);
		background: var(--block-background-fill);
	}

	.default.last.svelte-1kuiw39 {
		border-bottom: none;
	}

	.description.svelte-1kuiw39 {
		padding: 0.7rem 1rem;
		font-size: var(--scale-00);
		font-family: var(--font-sans);
		background: var(--block-background-fill);
	}

	.param.svelte-1kuiw39 {
		border-bottom: 1px solid var(--table-border-color);
	}

	.param.svelte-1kuiw39:last-child {
		border-bottom: none;
	}

	details[open].svelte-1kuiw39 .type:where(.svelte-1kuiw39) {
		border-bottom-width: 1px;
	}

	.param.md.svelte-1kuiw39 code:where(.svelte-1kuiw39) {
		background: none;
	}

	details.svelte-1kuiw39 > summary:where(.svelte-1kuiw39) {
		cursor: pointer;
	}

	details.svelte-1kuiw39 > summary:where(.svelte-1kuiw39)::-webkit-details-marker {
		display: none;
	}

	.param-link.svelte-1kuiw39 {
		opacity: 0;
		position: absolute;
		left: 8px;
		top: 50%;
		transform: translateY(-50%);
		transition: opacity 0.2s;
		color: var(--body-text-color);
		text-decoration: none;
	}

	.link-icon.svelte-1kuiw39 {
		font-size: 14px;
	}

	.type.svelte-1kuiw39:hover .param-link:where(.svelte-1kuiw39) {
		opacity: 0.7;
	}

	.param-link.svelte-1kuiw39:hover {
		opacity: 1 !important;
	}

	.param-content.svelte-1kuiw39 {
		overflow-y: auto;
	}