
	.overlay.svelte-17x1mvh {
		position: absolute;
		background-color: rgba(0, 0, 0, 0.4);
		width: 100%;
		height: 100%;
	}

	.hidden.svelte-17x1mvh {
		display: none;
	}

	.load-wrap.svelte-17x1mvh {
		display: flex;
		justify-content: center;
		align-items: center;
		height: 100%;
	}

	.loader.svelte-17x1mvh {
		display: flex;
		position: relative;
		background-color: var(--border-color-accent-subdued);
		animation: svelte-17x1mvh-shadowPulse 2s linear infinite;
		box-shadow:
			-24px 0 var(--border-color-accent-subdued),
			24px 0 var(--border-color-accent-subdued);
		margin: var(--spacing-md);
		border-radius: 50%;
		width: 10px;
		height: 10px;
		scale: 0.5;
	}

	@keyframes svelte-17x1mvh-shadowPulse {
		33% {
			box-shadow:
				-24px 0 var(--border-color-accent-subdued),
				24px 0 #fff;
			background: #fff;
		}
		66% {
			box-shadow:
				-24px 0 #fff,
				24px 0 #fff;
			background: var(--border-color-accent-subdued);
		}
		100% {
			box-shadow:
				-24px 0 #fff,
				24px 0 var(--border-color-accent-subdued);
			background: #fff;
		}
	}