import './i18n-CMo5lpzy.js';
import { a as append, f as from_svg } from './index-BvBk1Iap.js';
import './ScrollFade.svelte_svelte_type_style_lang-Xm-PLg9E.js';

var root = from_svg(`<svg class="dropdown-arrow svelte-loyhyk" xmlns="http://www.w3.org/2000/svg" width="100%" height="100%" viewBox="0 0 18 18"><path d="M5 8l4 4 4-4z"></path></svg>`);

function DropdownArrow($$anchor) {
	var svg = root();

	append($$anchor, svg);
}

export { DropdownArrow as D };
//# sourceMappingURL=DropdownArrow-Dw1NZKCM.js.map
