
:where(._jsonList.svelte-zxj9ye) {
  list-style: none;
  margin: 0;
  padding: 0;
  padding-left: var(--jsonPaddingLeft, 1rem);
  border-left: var(--jsonBorderLeft, 1px dotted);
}
:where(._jsonBkt.svelte-zxj9ye) {
  color: var(--jsonBracketColor, currentcolor);
}
.svelte-zxj9ye:where(._jsonBkt:where(.svelte-zxj9ye)):not(.empty):hover {
  cursor: pointer;
  background: var(--jsonBracketHoverBackground, #e5e7eb);
}
:where(._jsonSep.svelte-zxj9ye) {
  color: var(--jsonSeparatorColor, currentcolor);
}
:where(._jsonKey.svelte-zxj9ye) {
  color: var(--jsonKeyColor, currentcolor);
}
:where(._jsonVal.svelte-zxj9ye) {
  color: var(--jsonValColor, #9ca3af);
}
:where(._jsonVal:where(.svelte-zxj9ye)).string.svelte-zxj9ye {
  color: var(--jsonValStringColor, #059669);
}
:where(._jsonVal:where(.svelte-zxj9ye)).number.svelte-zxj9ye {
  color: var(--jsonValNumberColor, #d97706);
}
:where(._jsonVal:where(.svelte-zxj9ye)).boolean.svelte-zxj9ye {
  color: var(--jsonValBooleanColor, #2563eb);
}