
	ul.svelte-14aa7hi {
		white-space: nowrap;
		max-height: 100px;
		list-style: none;
		padding: 0;
		margin: 0;
	}

	.extra.svelte-14aa7hi {
		text-align: center;
	}

	.gallery.svelte-14aa7hi {
		align-items: center;
		cursor: pointer;
		padding: var(--size-1) var(--size-2);
		text-align: left;
	}