
	.wrap.svelte-16f20a1 {
		display: inline-block;
		width: var(--size-full);
		max-width: var(--size-full);
		color: var(--body-text-color);
	}

	.hide.svelte-16f20a1 {
		display: none;
	}

	.gallery.svelte-16f20a1 {
		display: flex;
		flex-wrap: wrap;
		gap: var(--spacing-lg);
	}

	.gallery-item.svelte-16f20a1 {
		border: 1px solid var(--border-color-primary);
		border-radius: var(--button-large-radius);
		overflow: hidden;
	}

	.gallery-item.svelte-16f20a1:hover {
		border-color: var(--border-color-accent);
		background: var(--table-row-focus);
	}

	.table-wrap.svelte-16f20a1 {
		border: 1px solid var(--border-color-primary);
		border-radius: var(--table-radius);
		width: var(--size-full);
		table-layout: auto;
		overflow-x: auto;
		line-height: var(--line-sm);
		color: var(--table-text-color);
	}
	table.svelte-16f20a1 {
		width: var(--size-full);
	}

	.tr-head.svelte-16f20a1 {
		box-shadow: var(--shadow-drop-lg);
		border-bottom: 1px solid var(--border-color-primary);
	}

	.tr-head.svelte-16f20a1 > :where(.svelte-16f20a1) + :where(.svelte-16f20a1) {
		border-right-width: 0px;
		border-left-width: 1px;
		border-color: var(--border-color-primary);
	}

	th.svelte-16f20a1 {
		padding: var(--size-2);
		white-space: nowrap;
	}

	.tr-body.svelte-16f20a1 {
		cursor: pointer;
		border-bottom: 1px solid var(--border-color-primary);
		background: var(--table-even-background-fill);
	}

	.tr-body.svelte-16f20a1:last-child {
		border: none;
	}

	.tr-body.svelte-16f20a1:nth-child(odd) {
		background: var(--table-odd-background-fill);
	}

	.tr-body.svelte-16f20a1:hover {
		background: var(--table-row-focus);
	}

	.tr-body.svelte-16f20a1 > :where(.svelte-16f20a1) + :where(.svelte-16f20a1) {
		border-right-width: 0px;
		border-left-width: 1px;
		border-color: var(--border-color-primary);
	}

	.tr-body.svelte-16f20a1:hover > :where(.svelte-16f20a1) + :where(.svelte-16f20a1) {
		border-color: var(--border-color-accent);
	}

	td.svelte-16f20a1 {
		padding: var(--size-2);
		text-align: center;
	}

	.paginate.svelte-16f20a1 {
		display: flex;
		justify-content: center;
		align-items: center;
		gap: var(--spacing-sm);
		margin-top: var(--size-2);
		color: var(--block-label-text-color);
		font-size: var(--text-sm);
	}

	button.current-page.svelte-16f20a1 {
		font-weight: var(--weight-bold);
	}
	.label.svelte-bnxc4d {
		display: flex;
		align-items: center;
		margin-bottom: var(--size-2);
		color: var(--block-label-text-color);
		font-weight: var(--block-label-text-weight);
		font-size: var(--block-label-text-size);
		line-height: var(--line-sm);
	}

	svg.svelte-bnxc4d {
		margin-right: var(--size-1);
	}